/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Locale;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;

import de.bernhardjung.dsaprogramm.basis.Kulturkunde;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.zauber.Zauber;


/** Enthaelt die main-Methoden die das Programm startet. */
public class Start {

    /** Zehn Sekunden in Millisekunden. */
    private static final int ZEHN_SEKUNDEN = 10 * 1000;

    /**
     * Startet das Programm.
     *
     * @param args momentan noch ungenutzt.
     */
    public static void main( final String[] args ) {

        // So kann man die Exceptions abfangen, die im Event Dispatch Thread
        // ausgeloest werden.
        // Das ist nichts offizielles, aber es funktioniert.
        // Leider funktioniert es nicht ueber Java-Webstart. :-(
        //System.setProperty( "sun.awt.exception.handler",
          //  "de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung" );

        try {

            String laF = UIManager.getSystemLookAndFeelClassName();
            
            if ( System.getProperty( "os.name" ).contains( "Linux" ) ) {

                // Unter Linux auf jeden Fall GTK benutzen.
                // Wenn kein Gnome Desktop laeuft wird als System L&F das
                // Standard Java L&F geliefert.
                laF = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
                //laF = "org.freeasinspeech.kdelaf.KdeLAF";
                // PlasticLookAndFeel.setPlasticTheme(new ExperienceBlue());
                // laF = "com.jgoodies.looks.plastic.PlasticXPLookAndFeel";
            }
            
            // UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
            UIManager.setLookAndFeel( laF );

        } catch ( final Exception e ) {
            // ignorieren. Zur Not tut es auch das Standard Look & Feel.
        }

        Locale.setDefault( Einstellungen.getInstanz().getSprache() );
        
        ToolTipManager.sharedInstance().setDismissDelay( ZEHN_SEKUNDEN );

        Basiswert.initialisieren();
        Eigenschaft.initialisieren();
        Talent.initialisieren();
        Kulturkunde.initialisieren();
        Sonderfertigkeit.initialisieren();
        VorNachteil.initialisieren();
        Zauber.initialisieren();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        try {

            speicher.ladeDaten();

            Rasse.LEERE_RASSE.setKulturen( speicher.getKulturen() );

            SwingUtilities.invokeLater( new Runnable() {

                public void run() {

                    final JFrame frame = new JFrame( I18n.i18n( "Programm.Name" ) );
                    
                    final Hauptpanel panel = new Hauptpanel( frame, frame );

                    frame.setIconImage( Util.ladeIcon( "icon" ).getImage() );

                    frame.setLocationByPlatform( true );

                    frame.setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );
                    frame.addWindowListener( new WindowAdapter() {
                        @Override
                        public void windowClosing(final WindowEvent e) {
                            panel.getActionBeenden().actionPerformed( null );
                        }
                    } );

                    frame.setJMenuBar( panel.erzeugeMenue( true ) );

                    frame.add( panel );

                    final Einstellungen einstellungen = Einstellungen.getInstanz();

                    frame.setSize( einstellungen.getHauptfensterGroesse() );
                    
                    frame.addComponentListener ( new ComponentAdapter() {
                        @Override
                        public void componentResized(ComponentEvent e) {
                            einstellungen.setHauptfensterGroesse( frame.getSize() );
                        }
                    } );

                    frame.setVisible( true );
                }
            } );

        } catch( final Exception e ) {

            DialogFehlermeldung.zeigeDialog(
                    "Der Datenspeicher konnte nicht initalisiert werden.", e );
        }

    }
}
