/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.namen;

import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.ATTR_BEZEICHNUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.ATTR_ID;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.ATTR_SCHLUESSEL;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMAT;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_MAENNLICH;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_MAENNLICH_ADLIG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_WEIBLICH;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_FORMATE_NAME_WEIBLICH_ADLIG;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_NAMENLISTE;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_NAMENLISTEN;
import static de.bernhardjung.dsaprogramm.daten.xml.NamenFactoryXmlTags.TAG_NAME_FACTORY;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

public class NameFactory implements Comparable< NameFactory > {

    /**
     * Das Suffix, das an die Id der Factory angehaengt wird um den Schluessel
     * zu erzeugen mit dem die Bezeichnung der Factory aus der 
     * Lokalisierungsdatei geladen werden kann. 
     */
    private static final String SUFFIX_BEZEICHNUNG = ".Bezeichnung";
    
    /**
     * Die Bezeichnung fuer die Factory. Sie gibt an welche Art von Namen von
     * der Factory erzeugt werden, z.B. "Garethische Namen".
     */
    private String bezeichnung;

    /**
     * Der Format-String, der verwendet wird um Namen von gemeinen Maennern zu
     * erzeugen.
     */
    private List< String > formatStringsNameMaennlich;

    /**
     * Der Format-String, der verwendet wird um Namen von adligen Maennern zu
     * erzeugen.
     */
    private List< String > formatStringsNameMaennlichAdlig;

    /**
     * Der Format-String, der verwendet wird, um Namen von gemeinen Frauen zu
     * erzeugen.
     */
    private List< String > formatStringsNameWeiblich;

    /**
     * Der Format-String, der verwendet wird um Namen von adligen Frauen zu
     * erzeugen.
     */
    private List< String > formatStringsNameWeiblichAdlig;

    /**
     * Die Id, die verwendet wird, um die Factory zu identifizieren.
     */
    private String id;

    /**
     * Die Namenlisten. Jede Liste ist unter dem Schluessel gespeichert, der
     * druch einen Namen aus der Liste ersetzt wird.
     */
    private Map< String, Namenliste > namenlisten;

    /**
     * Erzeugt eine Factory mit der uebergebenen Id.
     * 
     * @param dieId
     *            der Factory.
     */
    public NameFactory( final String dieId ) {
        
        id = dieId;

        final String schluesselBezeichnung = 
            getLokalisierungschluesselBezeichnung( id );
        
        if ( I18n.hatLokalisierungstext( schluesselBezeichnung ) ) {
            
            bezeichnung = I18n.i18n( schluesselBezeichnung );
        
        } else {
            
            bezeichnung = id;
        }
        
        formatStringsNameMaennlich = new ArrayList< String >( 1 );
        
        formatStringsNameMaennlichAdlig = new ArrayList< String >( 1 );
        
        formatStringsNameWeiblich = new ArrayList< String >( 1 );
        
        formatStringsNameWeiblichAdlig = new ArrayList< String >( 1 );
        
        namenlisten = new HashMap< String, Namenliste >();
    }
    
    /**
     * Sortiert die Factories nach Bezeichnung und Id.
     */
    public int compareTo( final NameFactory o ) {
        
        int result = Util.compare( getBezeichnung(), o.getBezeichnung() );
        
        if ( 0 != result ) {
            
            return result;
        }
            
        return getId().compareTo( o.getId() );
    }
/*
    public void einfuegen( final Namenliste namen ) {
        
        int index = Collections.binarySearch( namenlisten, namen );
        
        if ( 0 > index ) {
            
            index = - ( index + 1 );
            
            namenlisten.add( index, namen );
        
        } else {
            
            namenlisten.set( index, namen );
        }
    }
    
    public void entfernen( final Namenliste namen ) {
        
        int index = Collections.binarySearch( namenlisten, namen );
        
        if ( 0 <= index ) {
            
            namenlisten.remove( index );
        }   
    }
*/    
    @Override
    public int hashCode() {
        
        return id.hashCode();
    }

    @Override
    public boolean equals( Object obj ) {
        
        if ( ( obj == null )
                || ( getClass() != obj.getClass() ) )
        {
            return false;
        }
        
        final NameFactory factory = (NameFactory) obj;
        
        return getId().equals( factory.getId() );
    }

    /**
     * Ersetzt das erste Vorkommen des Schluessels in name mit dem 
     * Namenselement.
     * 
     * @param name
     * @param schluessel
     * @param namenselement
     */
    private void ersetzen( 
            final StringBuilder name,
            final String schluessel,
            final String namenselement ) {
        
        int start = name.indexOf( schluessel );
        
        if ( 0 > start )
        {
            return;
        }
        
        int ende = start + schluessel.length();
        
        name.replace( start, ende, namenselement );
    }

    /**
     * Erzeugt einen zufaelligen Namen fuer einen Helden.
     * 
     * @param held Der Held fuer den ein Name erzeugt werden soll.
     * @return Den Namen fuer den Helden.
     */
    public String erzeugeName( final Held held ) {
        
        final boolean istMaennlich = 
            Geschlecht.MAENNLICH == held.getGeschlecht();
        
        final boolean istAdlig = held.hat( VorNachteil.ADLIGE_ABSTAMMUNG );
        
        final StringBuilder name = new StringBuilder();
        
        if ( istMaennlich ) {
            
            if ( istAdlig 
                    && ( 0 < formatStringsNameMaennlichAdlig.size() ) ) {
                
                name.append( getString( formatStringsNameMaennlichAdlig ) );                
            }
            else {
                
                name.append( getString( formatStringsNameMaennlich ) );
            }   
        
        } else {
            
            if ( istAdlig 
                    && ( 0 < formatStringsNameWeiblichAdlig.size() ) ) {
                
                name.append( getString( formatStringsNameWeiblichAdlig ) );
            
            } else {
                
                name.append( getString( formatStringsNameWeiblich ) );
            }
        }
        
        for ( final String schluessel : namenlisten.keySet() ) {
            
            final Namenliste namen = namenlisten.get( schluessel );
            
            while ( 0 <= name.indexOf( schluessel ) ) {
                
                String namenselement = namen.getZufaelligenNamen();
                
                ersetzen( name, schluessel, namenselement );
            }
                
        }
        
        return name.toString();
    }
    
    /**
     * @return Die Bezeichnung der Factory.
     */
    public String getBezeichnung() {
        
        return bezeichnung;
    }
    
    /** @return Die Format-Strings fuer Namen gemeiner Maenner. */
    public List< String > getFormatStringsNamenMaennlich() {
        
        return new ArrayList< String >( formatStringsNameMaennlich );
    }

    /** @return Der Format-Strings fuer Namen adlige Maenner. */
    public List< String > getFormatStringsNamenMaennlichAdlig() {
        
        return new ArrayList< String >( formatStringsNameMaennlichAdlig );
    }

    /** @return Der Format-Strings fuer Namen gemeiner Frauen. */
    public List< String > getFormatStringsNamenWeiblich() {
        
        return new ArrayList< String >( formatStringsNameWeiblich );
    }

    /** @return Der Format-Strings fuer Namen adliger Frauen. */
    public List< String > getFormatStringsNamenWeiblichAdlig() {
        
        return formatStringsNameWeiblichAdlig;
    }
    
    /**
     * @return Die programmweit eindeutige Id der Factory.
     */
    public String getId() {
        
        return id;
    }
    
    /**
     * @param id Die Id der Factory.
     * @return Der Schluessel mit dem die Bezeichnung der Factory aus der 
     *         Lokalisierungsdatei geladen werden kann.
     */
    public String getLokalisierungschluesselBezeichnung( final String id ) {
        
        return id + SUFFIX_BEZEICHNUNG;
    }
    
    public Map< String, Namenliste > getNamenlisten() {
        
        return new HashMap< String, Namenliste >( namenlisten );
    }
    
    /**
     * Liefert zufaellig einen String aus der uebergebenen Liste.
     *  
     * @param formate Die Liste aus der ein String geliefert werden soll.
     * @return Einen der Strings oder einen leeren String, falls die Liste 
     *         leer ist. 
     */
    public String getString( final List< String > formate ) {
        
        if ( formate.isEmpty() ) {
            
            return "";
        }
        
        int index = new Random().nextInt( formate.size() );
        
        return formate.get( index );
    }

    /**
     * @param dieBezeichnung
     *            Legt den Namen der Factory fest. Ueberschreibt ggf. den 
     *            Lokalisierungstext.
     */
    public void setBezeichnung( final String dieBezeichnung ) {
        
        bezeichnung = dieBezeichnung;
    }
    
    /**
     * @param formate
     *            Das Format fuer Namen von gemeinen Maennern.
     */
    public void setFormatStringsNameMaennlich( final List< String > formate ) {
        
        formatStringsNameMaennlich.clear();
        
        formatStringsNameMaennlich.addAll( formate );
    }

    /**
     * @param formate
     *            Das Formate fuer Namen von adligen Maennern.
     */
    public void setFormatStringsNameMaennlichAdlig( 
            final List< String > formate ) {
        
        formatStringsNameMaennlichAdlig.clear();
        
        formatStringsNameMaennlichAdlig.addAll( formate );
    }

    /**
     * @param formate
     *            Das Formate fuer Namen von gemeinen Frauen.
     */
    public void setFormatStringsNameWeiblich( final List< String > formate ) {
        
        formatStringsNameWeiblich.clear();
        
        formatStringsNameWeiblich.addAll(  formate );
    }

    /**
     * @param formate
     *            Das Formate fuer Namen von adligen Frauen.
     */
    public void setFormatStringsNameWeiblichAdlig( 
            final List< String > formate ) {
        
        formatStringsNameWeiblichAdlig.clear();
        
        formatStringsNameWeiblichAdlig.addAll( formate );
    }
    
    /**
     * Legt die Namenlisten die benutzt werden, um Namen von Helden zu erzeugen.
     */
    public void setNamenlisten( final Map< String, Namenliste > neueListen ) {
        
        namenlisten.clear();
        
        namenlisten.putAll( neueListen );
    }
    
    /**
     * @return Die Bezeichnung der Factory.
     */
    @Override
    public String toString() {
        
        return bezeichnung;
    }
    
    /**
     * @return Eine XML-Repraesentation der Factory, die zum Speichern verwendet
     *         werden kann.
     */
    public IXMLElement toXml() {
        
        IXMLElement element = new XMLElement( TAG_NAME_FACTORY );
        
        final String schluesselBezeichnung =
            getLokalisierungschluesselBezeichnung( id );
        
        if ( ( ! I18n.hatLokalisierungstext( schluesselBezeichnung ) ) 
                || ( ! I18n.i18n( schluesselBezeichnung ).equals(bezeichnung))){
            
            element.setAttribute(
                    ATTR_BEZEICHNUNG,
                    bezeichnung );
        }

        final IXMLElement formateElement = new XMLElement( TAG_FORMATE );
        element.addChild( formateElement );
        
        formateElement.addChild( toXml( 
                TAG_FORMATE_NAME_MAENNLICH, 
                formatStringsNameMaennlich ) );

        formateElement.addChild( toXml(
                TAG_FORMATE_NAME_MAENNLICH_ADLIG,
                formatStringsNameMaennlichAdlig ) );

        formateElement.addChild( toXml(
                TAG_FORMATE_NAME_WEIBLICH,
                formatStringsNameWeiblich ) );

        formateElement.addChild( toXml(
                TAG_FORMATE_NAME_WEIBLICH_ADLIG,
                formatStringsNameWeiblichAdlig ) );
        
        element.setAttribute( ATTR_ID, id );
        
        final IXMLElement namenlistenElement = new XMLElement( TAG_NAMENLISTEN);
        element.addChild( namenlistenElement );
        
        for ( final String schluessel : namenlisten.keySet() ) {
            
            final IXMLElement namenlisteElement = 
                new XMLElement( TAG_NAMENLISTE );
            namenlistenElement.addChild( namenlisteElement );
            
            final Namenliste namenliste = namenlisten.get( schluessel );
            
            namenlisteElement.setAttribute( ATTR_ID, namenliste.getId() );
            
            namenlisteElement.setAttribute( ATTR_SCHLUESSEL, schluessel );
        }
        
        return element;
    }
    
    /**
     * Wandelt eine Liste von Formaten fuer Namen in eine XML-Repraesentation
     * um. Es wird ein XML-Element erzeugt, dass die Formate als Kindelemente
     * enthaelt.
     * 
     * @param tagName
     *            Der Name des XML-Elements das die Liste der Formate enthaelt
     * @param formate
     *            Die Formate, die gespeichert werden sollen.
     * @return  Die XML-Repaesentation der Formatliste.
     */
    public IXMLElement toXml(
            final String tagName,
            final List< String > formate ) {
        
        final IXMLElement formatElemente = new XMLElement( tagName );
        
        for ( final String format : formate ) {
            
            final IXMLElement formatElement = new XMLElement( TAG_FORMAT );
            formatElemente.addChild( formatElement );
            
            formatElement.setContent( format );
        }
        
        return formatElemente;
    }
}
