/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.BesondererBesitz;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungSo;

/**
 * Repraesentiert eine Professionen.
 *
 * @author Bernhard Jung
 */
public class Profession extends Heldenbaustein {

    /**
     * Eine Profession die keine Boni vergibt und keine Kosten verursacht.
     * Sie kann als Platzhalter eingesetzt werden wenn fuer einen Held noch
     * keine Profession gewaehlt wurde.
     */
    public static final Profession LEERE_PROFESSION =
        new Profession( "Keine Profession" );

    static {
        LEERE_PROFESSION.setVoraussetzungen(
                Arrays.asList( (Voraussetzung) new VoraussetzungSo( 0, 13 ) ));
    }

    /** Die Moeglichkeiten fuer den Vorteil Besonderer Besitz. */
    private final List< BesondererBesitz > besondererBesitz;

    /**
     * Gibt an ob es sich um eine Kaempferprofession handelt. Diese
     * Professionen koennen den Vorteil Viertelzauberer nicht waehlen.
     */
    private boolean kaempferprofession;

    /** Die Professionsbezeichnung fuer weibliche Helden. */
    private String nameWeiblich;

    /** Gibt an ob die Profession zeitaufwaendig ist. */
    private boolean zeitaufwaendig;

    /**
     * Erzeugt eine neue Profession und initialisiert alle Werte.
     *
     * @param id die Id der Profession.
     */
    public Profession( final String id ) {

        super( id, 25 );

        besondererBesitz = new ArrayList< BesondererBesitz >();

        kaempferprofession = false;

        nameWeiblich = id;

        zeitaufwaendig = false;
    }

    /**
     * Erzeugt das XML-Element, das die Namen der Profession beinhaltet.
     *
     * @return das passende XML-Element oder {@code null}, wenn die Namen aus
     *         der Lokalisierungsdatei geladen werden koennen.
     */
    private IXMLElement erzeugeNameElement() {

        IXMLElement nameElement = null;

        if ( nameNichtLokalisiert( ".Name.weiblich", nameWeiblich ) ) {

            nameElement = new XMLElement( TAG_NAME );

            final IXMLElement weiblichElement = new XMLElement( TAG_WEIBLICH );

            weiblichElement.setContent( nameWeiblich );

            nameElement.addChild( weiblichElement );
        }

        if ( nameNichtLokalisiert( ".Name.maennlich", getName() ) ) {

            if ( null == nameElement ) {

                nameElement = new XMLElement( TAG_NAME );
            }

            final IXMLElement maennlichElement = new XMLElement( TAG_MAENNLICH );

            maennlichElement.setContent( getName() );

            nameElement.addChild( maennlichElement );
        }

        return nameElement;
    }

    /** {@inheritDoc} */
    @Override
    IXMLElement erzeugeXmlWurzel() {
        return new XMLElement( TAG_PROFESSION );
    }

    /**
     * @return die Moeglichkeiten fuer den Vorteil Besonderer Besitz.
     */
    public final List< String > getBesondererBesitz() {

        final List< String > strings =
            new ArrayList< String >( besondererBesitz.size() );

        for ( final BesondererBesitz besitz : besondererBesitz ) {

            strings.add( besitz.getText() );
        }

        return strings;
    }
    /**
     * @return die Moeglichkeiten fuer den Vorteil Besonderer Besitz.
     */
    public final List< BesondererBesitz > getBesondererBesitzObjekte() {

        return new ArrayList< BesondererBesitz >( besondererBesitz );
    }

    /**
     * @return die zum Geschlecht des Helden passende Bezeichnung der
     *      Profession.
     */
    public final String getName( final Held held ) {

        if ( Geschlecht.MAENNLICH == held.getGeschlecht() ) {

            return getName();

        } else {

            return nameWeiblich;

        }
    }

    /** @return der Name der Profession fuer weibliche Charaktere. */
    public final String getNameWeiblich() {
        return nameWeiblich;
    }

    /**
     * @return {@code true}, wenn es sich um eine Kaempferprofession handelt,
     *         die den Vorteil Viertelzauberer nicht waehlen darf, sonst
     *         {@code false}.
     */
    public final boolean istKaempferprofession() {
        return kaempferprofession;
    }

    /**
     * @return {@code true}, wenn die Profession Zeitaufwaendig ist, sonst
     *         {@code false}.
     */
    public final boolean istZeitaufwaendig() {
        return zeitaufwaendig;
    }

    /**
     * @param besitz
     *            die moeglichen Varianten fuer den Vorteil Besonderer Besitz.
     */
    public final void setBesondererBesitz( final List< BesondererBesitz > besitz ) {

        besondererBesitz.clear();

        besondererBesitz.addAll( besitz );
    }

    /**
     * @param kaempfer
     *            {@code true}, wenn es sich um eine Kaempferprofession
     *            handelt, die den Vorteil Viertelzauberer nicht waehlen darf,
     *            sonst {@code false}.
     */
    public final void setKaempferprofession( final boolean kaempfer ) {

        kaempferprofession = kaempfer;
    }

    /**
     * Legt die Bezeichnung der Profession fuer Heldinnen fest.
     *
     * @param neuerName der Profession.
     */
    public final void setNameWeiblich( final String neuerName ) {
        nameWeiblich = neuerName;
    }

    /**
     * @param isZeitaufwaendig
     *            {@code true}, wenn die Profession zeitaufwaendig ist, sonst
     *            {@code false}.
     */
    public final void setZeitaufwaendig( final boolean isZeitaufwaendig ) {
        zeitaufwaendig = isZeitaufwaendig;
    }

    @Override
    public IXMLElement toXml() {

        final IXMLElement wurzelElement = super.toXml();

        if ( kaempferprofession ) {

            wurzelElement.setAttribute( ATTR_KAEMPFERPROFESSION, "true" );
        }

        if ( zeitaufwaendig ) {

            wurzelElement.setAttribute( ATTR_ZEITAUFWAENDIG, "true" );
        }

        // Der Name
        // ********
        final IXMLElement nameElement = erzeugeNameElement();
        if ( null != nameElement ) {

            wurzelElement.addChild( nameElement );
        }

        // Besonderer Besitz
        // *****************
        if ( 0 != besondererBesitz.size() ) {

            final IXMLElement besondererBesitzElement =
                new XMLElement( TAG_BESONDERER_BESITZ );

            for ( final BesondererBesitz besitz : besondererBesitz ) {

                besondererBesitzElement.addChild( besitz.toXml() );
            }

            wurzelElement.addChild( besondererBesitzElement );
        }

        return wurzelElement;
    }
}
