/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten;

import static de.bernhardjung.dsaprogramm.daten.Datenkonstanten.DATEIERWEITERUNG_RASSE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.IXMLParser;
import net.n3.nanoxml.IXMLReader;
import net.n3.nanoxml.StdXMLReader;
import net.n3.nanoxml.XMLParserFactory;
import de.bernhardjung.dsaprogramm.basis.Wuerfel;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.util.AlterFormel;
import de.bernhardjung.dsaprogramm.bausteine.util.GewichtFormel;
import de.bernhardjung.dsaprogramm.bausteine.util.GroesseFormel;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactoryNameIn;
import de.bernhardjung.dsaprogramm.daten.factorys.bausteine.FactoryWuerfeltabelleIn;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;

/**
 * Laedt die Rassen.
 *
 * @author Bernhard Jung
 */
public class RassenLader extends HeldenbausteinLader< Rasse > {

    /** Die XML-Dateien mit den Rassen in der Jar-Datei */
    private static final String[] RASSEN_DATEIEN = {
        "Elfen.ras",
        "Halbelfen.ras",
        "Mittellaender.ras",
        "Thorwaler.ras",
        "Tulamiden.ras",
        "Zwerge.ras"
    };

    /** Der Parser zum Laden der XML-Dateien. */
    private final IXMLParser parser;

    /** Erzeugt den Namen der Rasse aus dem XML-Dokument. */
    private final FactoryNameIn name;

    /** Erzeugt eine Wuerfeltabelle fuer die Augen- oder Haarfarbe. */
    private final FactoryWuerfeltabelleIn wuerfeltabelle;

    /**
     * @throws Exception
     *             wenn es zu einem Fehler beim Initialisieren des XML-Parsers
     *             kommt.
     */
    public RassenLader() throws Exception {

        parser = XMLParserFactory.createDefaultXMLParser();

        name = new FactoryNameIn();

        wuerfeltabelle = new FactoryWuerfeltabelleIn();
    }

    /**
     * Erzeugt die Formel zum Auswuerfeln des Alters eines Helden.
     *
     * @param wurzelElement des XML-Datei.
     * @return die Formel zum Auswuerfeln des Alters eines Helden.
     */
    private final AlterFormel erzeugeAlter( final IXMLElement wurzelElement ) {

        final IXMLElement alterElement;
        alterElement = wurzelElement.getFirstChildNamed( TAG_ALTER );

        final String basisString = alterElement.getAttribute( ATTR_BASIS );
        final int basis = Integer.valueOf( basisString );

        final String wuerfelString;
        wuerfelString = alterElement.getAttribute( ATTR_WUERFEL );

        final int wuerfel = Integer.valueOf( wuerfelString );

        return new AlterFormel( basis, new Wuerfel( wuerfel ) );
    }

    /**
     * @param wurzelElement der XML-Datei.
     * @return die aus der XML-Datei ermittelte Formel fuer das Gewicht eines
     *      Helden.
     */
    private final GewichtFormel erzeugeGewicht( final IXMLElement wurzelElement ) {

        final IXMLElement abzugElement;
        abzugElement = wurzelElement.getFirstChildNamed( TAG_GEWICHT );

        final int abzug = Integer.valueOf( abzugElement.getContent() );

        return new GewichtFormel( abzug );
    }

    /**
     * @param wurzelElement der XML-Datei.
     * @return die aus der XMl-Datei ermittelte Formel fuer die Groesse eines
     *      Helden.
     */
    private final GroesseFormel erzeugeGroesse( final IXMLElement wurzelElement ) {

        final IXMLElement groesse;
        groesse = wurzelElement.getFirstChildNamed( TAG_KOERPERGROESSE );

        final int basis;
        basis = Integer.valueOf( groesse.getAttribute( ATTR_BASIS ) );

        final int w20;
        w20 = Integer.valueOf( groesse.getAttribute( ATTR_W20 ) );

        final int w6;
        w6 = Integer.valueOf( groesse.getAttribute( ATTR_W6 ) );

        return new GroesseFormel( basis, w20, w6 );
    }

    /**
     * Ermittelt die Liste der moeglichen Kulturen der Rasse.
     *
     * @param wurzelElement der XML-Datei.
     * @return eine Liste der moeglichen Kulturen der Rasse.
     */
    private final Collection< Kultur > erzeugeKulturen(
            final IXMLElement wurzelElement ) {

        final IXMLElement kulturenElement;
        kulturenElement = wurzelElement.getFirstChildNamed( TAG_KULTUREN );

        final Collection< Kultur > kulturen = new LinkedList< Kultur >();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        for ( final IXMLElement kultur : kulturenElement.getChildren() ) {

            final String id = kultur.getContent();

            kulturen.add( speicher.getKultur( id ) );
        }

        return kulturen;
    }

    /**
     * @param wurzelElement der geladenen XML-Datei.
     * @return die aus der XML-Datei erzeugte Rasse.
     */
    private final Rasse erzeugeRasse(
            final IXMLElement wurzelElement ) {

        final Rasse rasse = erzeugeBaustein( wurzelElement );

        rasse.setName( name.erzeugen( wurzelElement, rasse.getId() ) );

        rasse.setAlter( erzeugeAlter( wurzelElement ) );

        rasse.setAugenfarben( wuerfeltabelle.erzeugen(
                wurzelElement.getFirstChildNamed( TAG_AUGENFARBEN ) ) );

        rasse.setHaarfarben( wuerfeltabelle.erzeugen(
                wurzelElement.getFirstChildNamed( TAG_HAARFARBEN ) ) );

        rasse.setGroesse( erzeugeGroesse( wurzelElement ) );

        rasse.setGewicht( erzeugeGewicht( wurzelElement ) );

        rasse.setKulturen( erzeugeKulturen( wurzelElement ) );

        return rasse;
    }

    /* *
     * Speichert alles Rassen in der jar-Datei in ein Verzeichnis.
     *
     * @param verzeichnis
     *            in das die Rassen gespeichert werden sollen.
     * /
    public final void exportieren( final File verzeichnis ) {

        verzeichnis.mkdir();

        for ( final String datei : RASSEN_DATEIEN ) {

            final String pfad = "rassen/" + datei;

            BufferedInputStream in = null;

            BufferedOutputStream out = null;

            try {

                in = new BufferedInputStream(
                        RassenLader.class.getResourceAsStream( pfad ) );

                out = new BufferedOutputStream(
                            new FileOutputStream(
                                    new File( verzeichnis, datei ) ) );

                final byte[] buffer = new byte[1024];

                int length;

                while ( ( length = in.read( buffer ) ) > 0 ) {

                    out.write( buffer, 0, length );
                }

            } catch( final IOException e ) {

                DialogFehlermeldung.zeigeDialog(
                        "Die Datei " + datei
                        + " konnte nicht exportiert werden.", e );

            } finally {

                if ( null != in ) {
                    try {
                        in.close();
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }

                if ( null != out ) {
                    try {
                        out.close();
                    } catch (final IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
*/
    @Override
    protected Rasse getBaustein( final String id ) {
        return Datenspeicher.getInstanz().getRasse( id );
    }

    /**
     * Laedt Rassen aus der Jar-Datei.
     *
     * @return die aus der Jar-Dateien geladenen Rassen.
     */
    public final List< Rasse > ladeAusJar() {

        final List< Rasse > rassen;
        rassen = new ArrayList< Rasse >( RASSEN_DATEIEN.length );

        for ( final String datei : RASSEN_DATEIEN ) {

            final String pfad = "rassen/" + datei;

            final InputStream in;
            in = RassenLader.class.getResourceAsStream( pfad );

            if ( null == in ) {

                DialogFehlermeldung.zeigeDialog( String.format(
                        "Die Rasse %1$s konnte nicht gefunden werden.",
                        pfad ) );

                continue;
            }

            try {

                final Rasse rasse = ladeRasse( datei, in );

                rassen.add( rasse );

            } catch (final Exception e) {

                DialogFehlermeldung.zeigeDialog( String.format(
                        "Die Datei %1$s konnte nicht aus der Jar-Datei " +
                        "geladen werden.", datei ),
                        e );
            }
        }

        return rassen;
    }

    /**
     * Laedt alle Rassen aus einem Verzeichnis.
     *
     * @param verzeichnis das die Rassen enthaelt.
     * @return die Liste der geladenen Rassen.
     */
    public final List< Rasse > laden( final File verzeichnis ) {

        final List< Rasse > rassen = new ArrayList< Rasse >();

        for ( final File datei : verzeichnis.listFiles() ) {

            final String name = datei.getName();

            if ( name.toLowerCase().endsWith( DATEIERWEITERUNG_RASSE ) ) {

                try {

                    final InputStream in = new FileInputStream( datei );

                    final Rasse rasse = ladeRasse( name, in );

                    rassen.add( rasse );

                } catch (final Exception e) {

                    DialogFehlermeldung.zeigeDialog( String.format(
                            "Die Datei %1$s konnte nicht " +
                            "geladen werden.", datei ),
                            e );
                }
            }
        }

        return rassen;
    }

    /**
     * @param dateiname
     *            der Datei, aus der die Rasse geladen werden soll.
     * @param in
     *            Inputstream der Datei, aus der die Rasse gelanden werden soll.
     * @return die aus der Datei geladene Rasse.
     * @throws Exception
     *             wenn es zu irgendeinem einem Fehler beim Laden der Datei
     *             kommt.
     */
    public final Rasse ladeRasse(
            final String dateiname,
            final InputStream in ) throws Exception {

        final IXMLReader reader = new StdXMLReader(
                new BufferedInputStream( in ) );

        parser.setReader( reader );

        final IXMLElement wurzelElement = (IXMLElement) parser.parse();

        return erzeugeRasse( wurzelElement );

    }

}
