/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.eigenschaften;

import java.util.HashMap;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Steigerbar;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Eine Instanz dieser Klasse repraesentiert eine Eigenschaft wie Mut
 * oder Klugheit. Sozialstatus wird im Programm ebenfalls als Eigenschaft
 * behandelt.
 *
 * @author Bernhard Jung
 */
public class Eigenschaft
implements Comparable< Eigenschaft >, Steigerbar {

    /**
     * Beschreibung die dem Benutzer erklaert, dass die Eigenschaft eine
     * Hoechststufe ueberschreitet, die durch einen Heldenbaustein vorgegeben
     * ist.
     */
    private static final String HELDENBAUSTEIN_HOECHSTSTUFE =
        I18n.i18n( "Eigenschaft.HeldenbausteinHoechststufe" );

    /**
     * Beschreibung die dem Benutzer erklaert, dass die Eigenschaft eine
     * Mindeststufe unterschreitet, die durch einen Heldenbaustein vorgegeben
     * ist.
     */
    private static final String HELDENBAUSTEIN_MINDESTSTUFE =
        I18n.i18n( "Eigenschaft.HeldenbausteinMindeststufe" );

    /**
     * Die maximale Anzahl an Punkten die bei der Erschaffung auf die
     * Eigenschaften verteilt werden darf.
     */
    private static final int MAX_VERTEILBAR = 100;

    /**
     * Beschreibt dem Benutzer, dass maximal Startstufe/2 zur Eigenschaft
     * hinzugekauft werden duerfen.
     */
    private static final String MAX_ZUKAUF =
        I18n.i18n( "Eigenschaft.MaxZukauf" );

    /**
     * Beschreibung die dem Benutzer erklaert, dass die Startstufe der
     * Eigenschaft zu hoch ist.
     */
    private static final String STARTSTUFE_ZU_HOCH =
        I18n.i18n( "Eigenschaft.StartstufeZuHoch" );

    /**
     * Beschreibung die dem Benutzer erklaert, dass die Startstufe der
     * Eigenschaft zu niedrig ist.
     */
    private static final String STARTSTUFE_ZU_NIEDRIG =
        I18n.i18n( "Eigenschaft.StartstufeZuNiedrig" );

    /** Der Steigerungsfaktor fuer die Eigenschaften. */
    private static final int STEIGERUNGSFAKTOR = SKT.H;

    /**
     * Erklaert dem Benutzer, dass nicht mehr als 100 Punkte auf die
     * Eigenschaften verteilt werden duerfen.
     */
    private static final String ZU_VIEL_PUNKTE_VERTEILT =
        I18n.i18n( "Eigenschaft.zuVielPunkteVerteilt" );

    /** Eine Liste aller Eigenschaften ohne SO. */
    private static final Eigenschaft[] EIGENSCHAFTEN;

    /**
     * Eine Map in der jede Eigenschaft unter ihrer Id gespeichert ist.
     */
    private static final Map< String, Eigenschaft > EIGENSCHAFTEN_MAP =
        new HashMap< String, Eigenschaft >();

    /** Eine Liste aller Eigenschaften mit SO. */
    private static final Eigenschaft[] EIGENSCHAFTEN_MIT_SO;

    /** Die Eigenschaft Mut. */
    public static final Eigenschaft MU = new Eigenschaft(
            "Eigenschaft.Mut", 0, 8, 14,
            VorNachteil.HERAUSRAGENDER_MU,
            VorNachteil.MISERABLER_MU );

    /** Die Eigenschaft Klugheit. */
    public static final Eigenschaft KL = new Eigenschaft(
            "Eigenschaft.Klugheit", 1, 8, 14,
            VorNachteil.HERAUSRAGENDE_KL,
            VorNachteil.MISERABLE_KL );

    /** Die Eigenschaft Intuition. */
    public static final Eigenschaft IN = new Eigenschaft(
            "Eigenschaft.Intuition", 2, 8, 14,
            VorNachteil.HERAUSRAGENDE_IN,
            VorNachteil.MISERABLE_IN );

    /** Die Eigenschaft Charisma. */
    public static final Eigenschaft CH = new Eigenschaft(
            "Eigenschaft.Charisma", 3, 8, 14,
            VorNachteil.HERAUSRAGENDES_CH,
            VorNachteil.MISERABLES_CH );

    /** Die Eigenschaft Fingerfertigkeit. */
    public static final Eigenschaft FF = new Eigenschaft(
            "Eigenschaft.Fingerfertigkeit", 4, 8, 14,
            VorNachteil.HERAUSRAGENDE_FF,
            VorNachteil.MISERABLE_FF );

    /** Die Eigenschaft Gewandtheit. */
    public static final Eigenschaft GE = new Eigenschaft(
            "Eigenschaft.Gewandtheit", 5, 8, 14,
            VorNachteil.HERAUSRAGENDE_GE,
            VorNachteil.MISERABLE_GE );

    /** Die Eigenschaft Konstitution. */
    public static final Eigenschaft KO = new Eigenschaft(
            "Eigenschaft.Konstitution", 6, 8, 14,
            VorNachteil.HERAUSRAGENDE_KO,
            VorNachteil.MISERABLE_KO );

    /** Die Eigenschaft Koerperkraft. */
    public static final Eigenschaft KK = new Eigenschaft(
            "Eigenschaft.Koerperkraft", 7, 8, 14,
            VorNachteil.HERAUSRAGENDE_KK,
            VorNachteil.MISERABLE_KK );

    /** Die Eigenschaft Sozialstatus. */
    public static final Eigenschaft SO = new Eigenschaft(
            "Eigenschaft.Sozialstatus", 8, 1, 13, null, null );

    static {

        EIGENSCHAFTEN_MIT_SO =
            new Eigenschaft[] { MU, KL, IN, CH, FF, GE, KO, KK, SO };

        EIGENSCHAFTEN =
            new Eigenschaft[] { MU, KL, IN, CH, FF, GE, KO, KK };

    }

    /**
     * @param id der gesuchten Eigenschaft.
     * @return die zur id gehoerende Eigenschaft.
     * @throws IllegalArgumentException wenn es keine Eigenschaft mit der
     *      uebergebenen Id gibt.
     */
    public static final Eigenschaft getEigenschaft( final String id ) {

        if ( EIGENSCHAFTEN_MAP.containsKey( id ) ) {

            return EIGENSCHAFTEN_MAP.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es existiert keine Eigenschaft mit der Id " + id );

        }
    }

    /**
     * @return eine Lister aller Eigenschaften (ohne SO) die so sortiert ist
     *      wie auf dem Charakterbogen. An Position 0 steht Mut, die letzte
     *      Eigenschaft ist KK (Position 7).
     */
    public static final Eigenschaft[] getEigenschaften() {

        return EIGENSCHAFTEN.clone();
    }

    /**
     * @return eine Lister aller Eigenschaften (plus SO) die so sortiert ist
     *      wie auf dem Charakterbogen. An Position 0 steht Mut, die letzten
     *      Eigenschaften sind KK (Position 7) und SO (Position 8)
     */
    public static final Eigenschaft[] getEigenschaftenMitSo() {

        final int length = EIGENSCHAFTEN_MIT_SO.length;

        final Eigenschaft[] kopie = new Eigenschaft[ length ];

        System.arraycopy( EIGENSCHAFTEN_MIT_SO, 0, kopie, 0, length );

        return kopie;
    }

    /**
     * @param held
     *            auf dessen Eigenschaften die Punkte verteilt werden sollen.
     * @return die maximale Anzahl an Punkten, die auf die Eigenschaften eines
     *         Helden verteilt werden duerfen.
     */
    public static final int getMaxVerteilbar( final Held held ) {

        int max = MAX_VERTEILBAR;

        final VorNachteil[] vorteile = { VorNachteil.HERAUSRAGENDE_FF,
                                         VorNachteil.HERAUSRAGENDE_GE,
                                         VorNachteil.HERAUSRAGENDE_IN,
                                         VorNachteil.HERAUSRAGENDE_KK,
                                         VorNachteil.HERAUSRAGENDE_KL,
                                         VorNachteil.HERAUSRAGENDE_KO,
                                         VorNachteil.HERAUSRAGENDER_MU,
                                         VorNachteil.HERAUSRAGENDES_CH };

        for ( final VorNachteil vorteil: vorteile ) {

            if ( held.hat( vorteil ) ) {

                max += held.getWert( vorteil ).getStufe();
            }
        }

        return max;
    }

    /**
     * Initialisiert die Eigenschaften vollstaendig.
     */
    public static final void initialisieren() {

        GE.einfuegenModifikation( VorNachteil.LAHM,      -2 );
        GE.einfuegenModifikation( VorNachteil.EINBEINIG, -5 );

        SO.einfuegenModifikation( VorNachteil.AKADEMISCHE_AUSBILDUNG_KRIEGER,
                                  1 );
    }

    /** Der Vorteil 'Herausragende Eigenschaft' fuer diese Eigenschaft. */
    private final VorNachteil herausragendeEigenschaft;

    /** Die ID der Eigenschaft. */
    private final String id;

    /**
     * zum sortieren der Eigenschaften, wie auf dem Heldenbogen. Die
     * Eigenschaften sind normalerweise nicht alphabetisch sortiert, sonder 1.
     * Mut, 2. Klugheit, 3. Intuition ...
     */
    private final int index;

    /**
     * Die Kurzform des Namens der Eigenschaft. <br>
     * Bsp.: "MU", "KL"...
     */
    private final String kuerzel;

    /**
     * Maximum fuer die Basisstufe der Eigenschaft. Das sind normalerweise die
     * GP, die auf die Eigenschaft verteilt wurden. Normalerweise 14, fuer SO
     * aber 13.
     */
    private final int maxBasisstufe;

    /**
     * Minimum fuer die Basisstufe der Eigenschaft. Das sind die GP die auf die
     * Eigenschaft verteilt werden muessen. Normalerweise 8, fuer SO aber 1.
     */
    private final int minBasisstufe;

    /** Der Nachteil 'Miserable Eigenschaft' fuer diese Eigenschaft. */
    private final VorNachteil miserableEigenschaft;

    /** Moegliche Modifikationen fuer die Eigenschaft. */
    private final Map< VorNachteil, Integer > modifikationen;

    /**
     * Der volle Name der Eigenschaft. <br>
     * Bsp.: "Mut", "Klugheit"..
     */
    private final String name;

    /**
     * Erzeugt eine neue Eigenschaft.
     *
     * @param derId
     *            der Eigenschaft
     * @param derIndex
     *            zum sortieren der Eigenschaft.
     * @param dieMinBasisstufe
     *            bei der Erschaffung.
     * @param dieMaxBasisstufe
     *            bei der Erschaffung.
     * @param dieHerausragendeEigenschaft
     *            Der Vorteil Herausragende Eigenschaft fuer diese Eigenschaft.
     * @param dieMiserableEigenschaft
     *            Der Nachteil Miserable Eigenschaft fuer diese Eigenschaft.
     */
    Eigenschaft(
            final String dieId,
            final int derIndex,
            final int dieMinBasisstufe,
            final int dieMaxBasisstufe,
            final VorNachteil dieHerausragendeEigenschaft,
            final VorNachteil dieMiserableEigenschaft  ) {

        herausragendeEigenschaft = dieHerausragendeEigenschaft;

        id = dieId;

        index = derIndex;

        kuerzel = I18n.i18n( dieId + ".Kuerzel" );

        maxBasisstufe = dieMaxBasisstufe;

        minBasisstufe = dieMinBasisstufe;

        miserableEigenschaft = dieMiserableEigenschaft;

        modifikationen = new HashMap< VorNachteil, Integer >();

        name = I18n.i18n( dieId + ".Name" );


        EIGENSCHAFTEN_MAP.put( id, this );
    }

    /**
     * Sortiert die Eigenschaften wie durch den Sortierindex vorgegeben.
     */
    public int compareTo( final Eigenschaft o ) {
        return getIndex() - o.getIndex();
    }

    /**
     * Fuegt der Eigenschaft eine Modifikation durch einen Vor-/Nachteil hinzu.
     * Wenn ein Held den Vor-/Nachteil besitzt wird seine Stufe in der
     * Eigenschaft um die Modifikation veraendert.
     *
     * @param vorNachteil
     *            durch den die Modifikation entsteht.
     * @param modifikation
     *            die auf die Eigenschaft vergeben wird.
     */
    private final void einfuegenModifikation(
            final VorNachteil vorNachteil,
            final int modifikation ) {

        modifikationen.put( vorNachteil, modifikation );
    }

    /** @return die Id der Eigenschaft. */
    public final String getId() {
        return id;
    }

    /**
     * @return den Sortierindex der Eigenschaft.
     */
    public final int getIndex() {
        return index;
    }

    /**
     * @return ein Kuerzel wie "MU"
     */
    public final String getKuerzel() {
        return kuerzel;
    }

    /**
     * Beachtet Herausragende Eigenschaft und Miserable Eigenschaft bei der
     * Bestimmung der Obergrenze fuer die Basisstufe der Eigenschaft.
     *
     * @param held
     *            fuer den die Obergrenze gelten soll.
     * @return der groesste noch erlaubte Wert fuer die Basisstufe der
     *         Eigenschaft.
     */
    public int getMaxBasisstufe( final Held held ) {

        int max = maxBasisstufe;

        if ( null != herausragendeEigenschaft ) {

            if ( held.hat( herausragendeEigenschaft ) ) {

                max += held.getWert( herausragendeEigenschaft ).getStufe();

            } else if ( held.hat( miserableEigenschaft ) ) {

                max = 8;

            }
        }

        return max;
    }

    /**
     * Bestimmt die minimale Basisstufe der Eigenschaft unter Beachtung von
     * Herausragender und Miserabler Eigenschaft.
     *
     * @param held
     *            fuer den die minimale Basisstufe gelten soll.
     * @return der kleinste noch erlaubte Wert fuer die Basisstufe der
     *         Eigenschaft.
     */
    public int getMinBasisstufe( final Held held ) {

        int min = minBasisstufe;

        if ( null != herausragendeEigenschaft ) {

            if ( held.hat( herausragendeEigenschaft ) ) {

                min = getMaxBasisstufe( held );

            } else if ( held.hat( miserableEigenschaft ) ) {

                min = 8;
            }

        }

        return min;
    }

    /** @return die Modifikation die ein Held auf die Eigenschaft erhaelt. */
    public int getModifikation( final Held held ) {

        int modifikation = 0;

        if ( held.getRasse().hatModifikation( this ) ) {

            modifikation += held.getRasse().getModifikation( this );
        }

        if ( held.getKultur().hatModifikation( this ) ) {

            modifikation += held.getKultur().getModifikation( this );
        }

        if ( held.getProfession().hatModifikation( this ) ) {

            modifikation += held.getProfession().getModifikation( this );
        }

        if ( held.hat( miserableEigenschaft ) ) {

            modifikation -= 1;
        }

        for ( final VorNachteil v : modifikationen.keySet() ) {

            if ( held.hat( v ) ) {

                modifikation += modifikationen.get( v );
            }
        }
        return modifikation;
    }

    /**
     * @return den Namen der Eigenschaft, z.B. "Mut".
     */
    public final String getName() {
        return name;
    }

    /** {@inheritDoc} */
    public final int getSteigerungsspalte() {
        return STEIGERUNGSFAKTOR;
    }

    /**
     * Prueft ob ein Wert die Hoechststufe eines Heldenbausteins ueberschreitet
     * und fuegt ggf eine entsprechende Problembeschreibung in die uebergebene
     * KonfliktBeschreibung ein.
     *
     * @param held um dessen Wert es geht.
     * @param wert der ueberprueft werden soll.
     * @param heldenbaustein der evtl die Hoechsttstufe verlangt.
     * @param beschreibung der evtl aufgetretenen Konflikte.
     */
    private final void pruefeBausteinHoechststufe(
            final EigenschaftWert wert,
            final Heldenbaustein heldenbaustein,
            final KonfliktBeschreibung beschreibung ) {

        if ( heldenbaustein.hatHoechststufe( this ) ) {

            final int hoechststufe = heldenbaustein.getHoechststufe( this );

            final int stufe = wert.getStufe();

            if ( stufe > hoechststufe ) {

                beschreibung.einfuegen( String.format(
                        HELDENBAUSTEIN_HOECHSTSTUFE,
                        heldenbaustein.getName(),
                        getKuerzel(),
                        hoechststufe ) );

            }
        }

    }

    /**
     * Prueft ob ein Wert die Mindeststufe eines Heldenbausteins unterschreitet
     * und fuegt ggf eine entsprechende Problembeschreibung in die uebergebene
     * KonfliktBeschreibung ein.
     *
     * @param held um dessen Wert es geht.
     * @param wert der ueberprueft werden soll.
     * @param heldenbaustein der evtl die Mindeststufe verlangt.
     * @param beschreibung der evtl aufgetretenen Konflikte.
     */
    private final void pruefeBausteinMindesstufe(
            final Held held,
            final EigenschaftWert wert,
            final Heldenbaustein heldenbaustein,
            final KonfliktBeschreibung beschreibung ) {

        if ( heldenbaustein.hatMindeststufe( held, this ) ) {

            final int mindeststufe = heldenbaustein.getMindeststufe( this );

            final int stufe = wert.getStufe();

            if ( stufe < mindeststufe ) {

                beschreibung.einfuegen( String.format(
                        HELDENBAUSTEIN_MINDESTSTUFE,
                        heldenbaustein.getName(),
                        getKuerzel(),
                        mindeststufe ) );

            }
        }

    }

    /**
     * Ueberprueft ob ein Eigenschafts-Wert fuer einen Helden regelkonform ist
     * und liefert eine entsprechende Konfliktbeschreibung zurueck.
     *
     * @param held gegen den der Wert getestet werden soll.
     * @param wert der ueberprueft werden soll.
     * @return eine Beschreibung der aufgetretenen Regelprobleme.
     */
    public KonfliktBeschreibung pruefen(
            final Held held,
            final EigenschaftWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.wirdErschaffen() ) {

            pruefeBausteinMindesstufe( held, wert, held.getRasse(), beschreibung );

            pruefeBausteinMindesstufe(
                    held,
                    wert,
                    held.getKultur(),
                    beschreibung );

            pruefeBausteinMindesstufe(
                    held,
                    wert,
                    held.getProfession(),
                    beschreibung );

            pruefeBausteinHoechststufe( wert, held.getProfession(), beschreibung );

            if ( wert.getBasisstufe() < getMinBasisstufe( held ) ) {

                beschreibung.einfuegen( String.format(
                        STARTSTUFE_ZU_NIEDRIG,
                        getMinBasisstufe( held ) ) );
            }

            if ( wert.getBasisstufe() > getMaxBasisstufe( held ) ) {

                beschreibung.einfuegen( String.format(
                        STARTSTUFE_ZU_HOCH,
                        getMaxBasisstufe( held ) ) );
            }

            final int verteilt = held.getVerteilteEigenschaftspunkte();

            final int verteilbar = getMaxVerteilbar( held );

            if ( ( this != SO ) && ( verteilbar < verteilt ) ) {

                beschreibung.einfuegen( String.format(
                        ZU_VIEL_PUNKTE_VERTEILT,
                        verteilbar ) );
            }

        } else {

            final int startstufe = wert.getStartstufe();

            final int startstufeHalbe = Math.round( startstufe / 2F );

            final int zukauf = wert.getZukauf();

            if ( zukauf > startstufeHalbe ) {

                beschreibung.einfuegen( String.format(
                        MAX_ZUKAUF, startstufeHalbe ) );
            }
        }

        return beschreibung;
    }

    /**
     * @return den Namen der Eigenschaft.
     */
    @Override
    public final String toString() {
        return getName();
    }
}
