/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Dimension;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JEditorPane;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeit;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteVariante;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Zauberbonus;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteil;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Strings;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Erlaubt es dem Benutzer einen Heldenbaustein zu waehlen. Auf der linken Seite
 * befindet sich eine Liste mit den Bausteinen und auf der rechten Seite werden
 * die Informationen zum gewaehlten Baustein angezeigt.
 *
 * @see PanelRasse
 * @see PanelKultur
 * @see PanelProfession
 * @see PanelRasKulProf
 *
 * @author Bernhard Jung
 */
abstract class PanelHeldenbaustein< T extends Heldenbaustein >
        extends JPanel
        implements ListSelectionListener, Observer {

    /**
     * Die Ueberschrift fuer die Aufzaehlung der automatischen Vor- und
     * Nachteile.
     */
    static final String AUTOMATISCHE_VOR_NACHTEILE =
        I18n.i18n( "PanelHeldenbaustein.AutomatischeVorNachteile" );

    /** Text mit dem die GP-Kosten der Rasse angezeigt werden. */
    static final String GENERIERUNGSKOSTEN =
        I18n.i18n( "PanelHeldenbaustein.Generierungskosten" );

    /**
     * Die Ueberschrift fuer die Aufzaehlung der empfohlenen Vor- und Nachteile.
     */
    static final String EMPFOHLENE_VOR_NACHTEILE =
        I18n.i18n( "PanelHeldenbaustein.EmpfohleneVorNachteile" );

    /** Das Kuerzel fuer Generierungspunkte z.B. 'GP'. */
    static final String GP = I18n.i18n( "PanelHeldenbaustein.Gp" );

    /**
     * Zeigt an, dass der Baustein keine Eigenschafts-Modifikation, Vorteile
     * oder aehnliches vergibt.
     */
    static final String KEINE = I18n.i18n( "PanelHeldenbaustein.Keine" );

    /**
     * Zeigt an, dass der Baustein keine Eigenschafts-Modifikation vergibt oder
     * keine empfohlenen Vor-/Nachteile hat.
     */
    static final String KEINE_BESTIMMTEN =
        I18n.i18n( "PanelHeldenbaustein.KeineBestimmten" );

    /**
     * Der Text der vor der Aufzaehlung der Leittalente steht, z.B.
     * 'Leittalente'
     */
    static final String LEITTALENTE =
        I18n.i18n( "PanelHeldenbaustein.Leittalente" );

    /**
     * Die Ueberschrift 'Modifikationen' fuer die Aufzaehlung der
     * Modifikationen eines Heldenbausteins.
     */
    static final String MODIFIKATIONEN =
        I18n.i18n( "PanelHeldenbaustein.Modifikationen" );

    /**
     * Die Ueberschrift 'Sonderfertigkeiten' fuer die Aufzaehlung der
     * Sonderfertigkeiten die der Heldenbaustein vergibt.
     */
    static final String SONDERFERTIGKEITEN =
        I18n.i18n( "PanelHeldenbaustein.Sonderfertigkeiten" );

    /**
     * Die Ueberschrift 'Talente' fuer die Aufzaehlung der Talent-Boni eines
     * Heldenbausteins.
     */
    static final String TALENTE =
        I18n.i18n( "PanelHeldenbaustein.Talente" );

    /**
     * Die Ueberschrift fuer die Aufzaehlung der ungeeigneten Vor- und
     * Nachteile.
     */
    static final String UNGEEIGNETE_VOR_NACHTEILE =
        I18n.i18n( "PanelHeldenbaustein.UngeeigneteVorNachteile" );

    /**
     * Die Ueberschrift 'verbilligte Sonderfertigkeiten' fuer die Aufzaehlung
     * der verbilligten Sonderfertigkeiten die der Heldenbaustein vergibt.
     */
    static final String VERBILLIGTE_SONDERFERTIGKEITEN =
        I18n.i18n( "PanelHeldenbaustein.verbilligteSonderfertigkeiten" );

    /**
     * Erklaert dem Benutzer dass der Held durch den Baustein Verbindungen im
     * Wert von SO x Punkten erhaelt.
     */
    static final String VERBINDUNGEN =
        I18n.i18n( "PanelHeldenbaustein.Verbindungen" );

    /**
     * Die Ueberschrift 'Voraussetzungen' fuer die Aufzaehlung der
     * Voraussetzungen eines Heldenbausteins.
     */
    static final String VORAUSSETZUNGEN =
        I18n.i18n( "PanelHeldenbaustein.Voraussetzungen" );

    /** Der Text fuer die Aufzaehlung der Zauber. */
    private static final String ZAUBER =
        I18n.i18n( "PanelHeldenbaustein.Zauber" );

    /** Die Liste die die Heldenbausteine darstellt. */
    private final JList bausteinliste;

    /** Der Held fuer den der Baustein ausgewaehlt werden soll. */
    protected Held held;

    /** Hier werden die Informationen ueber den Heldenbaustein angezeigt. */
    protected final JEditorPane informationen;

    PanelHeldenbaustein() {

        super( new FormLayout( "7dlu, F:D:G, 7dlu", "7dlu, F:D:G, 7dlu") );

        final JSplitPane splitPane = new JSplitPane();

        Datenspeicher.getInstanz().addObserver( this );

        bausteinliste = new JList( getBausteine() );
        bausteinliste.addListSelectionListener( this );
        bausteinliste.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        final JScrollPane scrollPane = new JScrollPane( bausteinliste );
        scrollPane.setPreferredSize( new Dimension( 150, 200 ) );
        splitPane.setLeftComponent( scrollPane );

        informationen = new JEditorPane();
        informationen.setContentType( "text/html" );
        informationen.setEditable( false );

        // Ein Hack um zu verhindern dass man die Informationen mittels
        // Markieren und Strg+C kopieren kann.
        informationen.setDisabledTextColor( getForeground() );
        informationen.setEnabled( false );
       /*
        informationen.setText( "<html><head><style>" +
                "body {font-family:sans-serif;} " +
                "</style></head><body></body></html>");
        */
        splitPane.setRightComponent( new JScrollPane( informationen ) );

        add( splitPane, new CellConstraints( 2, 2 ) );
    }

    /**
     * Meldet das Panel als Observer des Helden ab.
     */
    public final void abmeldenVonHeld() {
        if ( null != held ) {
            held.deleteObserver( this );
        }
    }

    /**
     * Aktualisiert die angezeigten Informationen ueber den gewaehlten Baustein.
     * @return der aktuell gewaehlte Baustein. {@code null} wenn kein Baustein
     *         gewaehlt ist.
     */
    @SuppressWarnings("unchecked")
    T aktualisiereInformationen() {

        final T baustein = (T) bausteinliste.getSelectedValue();

        if ( null != baustein ) {
            informationen.setText( getInformationen( baustein ) );
            informationen.setCaretPosition( 0 );
        }

        return baustein;
    }

    /**
     * Haengt eine Textrepraesentation der automatischen Vor-/Nachteile an eine
     * StringBuilder an. Wenn es keine automatischen Vor-/Nachteile gibt, so
     * macht die Methode nichts.
     *
     * @param informationen
     *            Der StringBuilder an den der Text gehaengt wird.
     * @param vorteilBoni
     *            die automatischen Vorteile.
     * @param nachteilBoni
     *            die automatischen Nachteile.
     * @param verbindungen
     *            die Punkte die auf Verbindungen verteilt werden duerfen.
     */
    final void appendAutomatischeVorNachteile(
            final StringBuilder informationen,
            final List< VorNachteilBonus > vorteilBoni,
            final List< VorNachteilBonus > nachteilBoni,
            final int verbindungen ) {

        if ( ( 0 != verbindungen )
          || ( 0 < vorteilBoni.size() )
          || ( 0 < nachteilBoni.size() )) {

            informationen.append( "<p><b>" );
            informationen.append( AUTOMATISCHE_VOR_NACHTEILE );
            informationen.append( ": </b>" );

            if (0 < vorteilBoni.size()) {

                informationen
                        .append( Strings.erzeugeAufzaehlung( vorteilBoni ) );
            }

            if (0 != verbindungen) {

                if (0 < vorteilBoni.size()) {

                    informationen.append( ", " );
                }

                informationen.append( String
                        .format( VERBINDUNGEN, verbindungen ) );
            }

            if (0 < nachteilBoni.size()) {

                if (( 0 < vorteilBoni.size() ) || ( 0 != verbindungen )) {

                    informationen.append( " / " );
                }

                informationen
                        .append( Strings.erzeugeAufzaehlung( nachteilBoni ) );
            }

            informationen.append( "</p>" );
        }

    }

    /**
     * Haengt eine Aufzaehlung der empfohlenen Vor- und Nachteile an einen
     * StringBuilder an. Macht nichts, wenn es keine empfohlenen Vor-/Nachteile
     * gibt.
     *
     * @param informationen Der StringBuilder an den der Text gehaengt wird.
     * @param empfohleneVorteile des Bausteins.
     * @param empfohleneNachteile des Bausteins.
     */
    final void appendEmpfohleneVorNachteile(
            final StringBuilder informationen,
            final List< VorNachteil > empfohleneVorteile,
            final List< VorNachteil > empfohleneNachteile ) {

        if ( ( 0 < empfohleneVorteile.size() )
          || ( 0 < empfohleneNachteile.size() )) {

            informationen.append( "<p><b>" );
            informationen.append( EMPFOHLENE_VOR_NACHTEILE );
            informationen.append( ": </b>" );

            if (0 < empfohleneVorteile.size()) {

                informationen.append( Strings
                        .erzeugeAufzaehlung( empfohleneVorteile ) );
            }

            if (0 < empfohleneNachteile.size()) {

                if (0 < empfohleneVorteile.size()) {

                    informationen.append( " / " );
                }

                informationen.append( Strings
                        .erzeugeAufzaehlung( empfohleneNachteile ) );
            }

            informationen.append( "</p>" );
        }

    }

    /**
     * Haengt eine Textrepraesentation der Modifikationen an einen StringBuilder
     * an.
     *
     * @param baustein
     *            dessen Modifikationen angezeigt werden sollen.
     * @param informationen
     *            Der StringBuilder an den der Text gehaengt wird.
     * @param modifikationen
     *            um die es geht.
     */
    final void appendModifikationen(
            final StringBuilder informationen,
            final List< Object > modifikationen,
            final Heldenbaustein baustein ) {

        if ( vergibtVollzauberer( baustein ) ) {

            // AE +12 durch Vollzauberer
            modifikationen.add( Basiswert.AE.getKuerzel() + " +12" );

            // MR +2 durch Vollzauberer
            modifikationen.add( Basiswert.MR.getKuerzel() + " +2" );
        }

        if ( vergibtViertelzauberer( baustein ) ) {

            // AE -6 durch Viertelzauberer
            modifikationen.add( Basiswert.AE.getKuerzel() + " -6" );
        }

        informationen.append( "<p><b>" );
        informationen.append( MODIFIKATIONEN );
        informationen.append( ":</b> " );

        if ( ! modifikationen.isEmpty() ) {

            informationen.append(
                    Strings.erzeugeAufzaehlung( modifikationen ) );

        } else {

            informationen.append( "-" );

        }
        informationen.append( "</p>" );

    }

    /**
     * Haengt eine Liste von Sonderfertigkeiten an einen StringBuilder an.
     * Macht nichts wenn die Liste der Sonderfertigkeiten leer ist.
     *
     * @param informationen Der StringBuilder an den der Text angehaengt wird.
     * @param sonderfertigkeiten
     */
    final void appendSonderfertigkeiten(
            final StringBuilder informationen,
            final List< SonderfertigkeitBonus > sonderfertigkeiten ) {

        if ( sonderfertigkeiten.size() > 0 ) {

            informationen.append( "<p><b>" );
            informationen.append( SONDERFERTIGKEITEN );
            informationen.append( ":</b> " );

            informationen.append( sonderfertigkeiten.get( 0 ) );

            for ( int i = 1, n = sonderfertigkeiten.size() ; i < n ; ++i ) {

                final SonderfertigkeitBonus bonus = sonderfertigkeiten.get( i );

                informationen.append( "; " );
                informationen.append( bonus );
            }

            informationen.append( "</p>" );
        }

    }

    /**
     * Haengt die Liste der ungeeigneten Vor- und Nachteile an einen
     * StringBuilder an. Macht nichts, wenn es keine ungeeigneten Vor-/Nachteile
     * gibt.
     *
     * @param informationen
     * @param ungeeigneteVorteile
     * @param ungeeigneteNachteile
     */
    final void appendUngeeigneteVorNachteile(
            final StringBuilder informationen,
            final List< UngeeigneterVorNachteil > ungeeigneteVorteile,
            final List< UngeeigneterVorNachteil > ungeeigneteNachteile ) {

        if ( ( 0 < ungeeigneteNachteile.size() )
          || ( 0 < ungeeigneteVorteile.size() ) ) {

            informationen.append( "<p><b>" );
            informationen.append( UNGEEIGNETE_VOR_NACHTEILE );
            informationen.append( ": </b>" );

            if ( 0 < ungeeigneteVorteile.size() ) {

                informationen.append(
                        Strings.erzeugeAufzaehlung( ungeeigneteVorteile ) );
            }

            if ( 0 < ungeeigneteNachteile.size() ) {

                if ( 0 < ungeeigneteVorteile.size() ) {

                    informationen.append( " / " );
                }

                informationen.append(
                        Strings.erzeugeAufzaehlung( ungeeigneteNachteile ) );
            }

            if ( ( 0 == ungeeigneteVorteile.size() )
              && ( 0 == ungeeigneteNachteile.size() ) ) {

                informationen.append( KEINE_BESTIMMTEN );
            }

            informationen.append( "</p>" );
        }

    }

    /**
     * Haengt eine List von verbilligten Sonderfertigkeiten an einen
     * StringBuilder an. Macht nichts wenn die Listen leer sind.
     *
     * @param informationen
     *            Der StringBuilder an den der Text angehaengt wird.
     * @param verbilligte
     *            Die verbilligten Sonderfertigkeiten.
     * @param verbilligteVarianten
     *            Die Sonderfertigkeiten die nur in einer Variante verbilligt
     *            sind.
     */
    final void appendVerbilligteSonderfertigkeiten(
            final StringBuilder informationen,
            final List< VerbilligteSonderfertigkeit > verbilligte,
            final List< VerbilligteVariante > verbilligteVarianten ) {

        if ( ( verbilligte.size() > 0 )
          || ( verbilligteVarianten.size() > 0 ) ) {

            informationen.append( "<p><b>" );
            informationen.append( VERBILLIGTE_SONDERFERTIGKEITEN );
            informationen.append( ": </b>" );

            informationen.append( Strings.erzeugeAufzaehlung( verbilligte ) );

            if ( ( verbilligte.size() > 0 )
                    && ( verbilligteVarianten.size() > 0 )) {
                informationen.append( ", " );
            }

            informationen.append(
                    Strings.erzeugeAufzaehlung( verbilligteVarianten ) );

            informationen.append( "</p>" );
        }

    }

    /**
     * Haengt eine Aufzaehlung der Voraussetzungen des Bausteins an einen
     * StringBuilder an. Wenn es keine Voraussetzungen gibt wird nichts gemacht.
     *
     * @param informationen
     *            Der StringBuilder an den die Daten angehaengt werden sollen.
     * @param voraussetzungen
     *            Die an den StringBuilder angehaengt werden sollen.
     */
    final void appendVoraussetzungen(
            final StringBuilder informationen,
            final List< Voraussetzung > voraussetzungen ) {

        if ( 0 < voraussetzungen.size() ) {

            informationen.append( "<p><b>" );
            informationen.append( VORAUSSETZUNGEN );
            informationen.append( ":</b> " );

            final String text =  Strings.erzeugeAufzaehlung( voraussetzungen );

            informationen.append( text );
            informationen.append( "</p>" );
        }

    }

    final void appendZauber(
            final StringBuilder informationen,
            final List< Zauberbonus > zauber ) {

        if ( 0 < zauber.size() ) {

            informationen.append( "<p><b>" );
            informationen.append( ZAUBER );
            informationen.append( ":</b> " );

            final String text =  Strings.erzeugeAufzaehlung( zauber );

            informationen.append( text );
            informationen.append( "</p>" );

        }
    }

    /**
     * Wird von {@link #update(Observable, Object)} aufgerufen, um zu
     * ueberpruefen ob sich die Liste der angezeigten Baustein geaendert hat.
     * Das Panel ist als Observer beim Datenspeicher angemeldet.
     *
     *
     * @param arg das Argument, das an die update-Methode uebergeben wurde.
     * @return {@code true}, wenn die Liste der Bausteine sich geaendert hat,
     *         sonst {@code false}.
     */
    abstract boolean bausteineGeaendert( final Object arg );

    /**
     * @return die Liste der Bausteine, die im Panel zur Auswahl stehen.
     */
    abstract T[] getBausteine();

    /**
     * @return ein HTML-Dokument mit den Informationen zum uebergebenen
     *      Heldenbaustein.
     */
    abstract String getInformationen( final T heldenbaustein );

    /**
     * Diese Methode wird aufgerufen wenn ein neuer Baustein ausgewaehlt wurde.
     * Sie dient dazu um dem Helden den Baustein hinzuzufuegen. In der
     * Unterklasse wird dann also entsprechend die Rasse, Kultur oder
     * Profession des Helden aktualisiert. Wird {@code null} uebergeben so soll
     * der leere Heldenbaustein hinzugefuegt werden.
     *
     * @see Rasse#LEERE_RASSE
     * @see Kultur#LEERE_KULTUR
     * @see Profession#LEERE_PROFESSION
     *
     * @param baustein der Baustein der gerade ausgewaehlt wurde.
     */
    abstract void setGewaehlterBaustein( final T baustein );

    /**
     * Aendert den Helden fuer den der Baustein gewaehlt wird.
     * Meldet das Panel als Observer beim Helden an.
     *
     * @see #abmeldenVonHeld()
     * @param neuerHeld fuer den der Baustein ausgewaehlt wird.
     */
    @SuppressWarnings( "unchecked" )
    void setHeld( final Held neuerHeld ) {

        held = neuerHeld;

        if ( null != held ) {

            held.addObserver( this );

            setGewaehlterBaustein( (T) bausteinliste.getSelectedValue() );
        }
    }

    /**
     * Bestimmt die Komponente die zur Darstellung der Liste mit den Bausteinen
     * verwendet wird.
     *
     * @param renderer der fuer die Darstellung der Liste verwendet wird.
     */
    final void setListCellRenderer( final ListCellRenderer renderer ) {
        bausteinliste.setCellRenderer( renderer );
    }

    /**
     * Wird aufgerufen wenn in der Liste ein Heldenbaustein angeklickt wird.
     * Sorgt dafuer das die Informationen zum Heldenbaustein angezeigt werden
     * und dem Helden der gewaehlte Baustein hinzugefuegt wird.
     */
    public final void valueChanged(final ListSelectionEvent e) {

        if ( e.getValueIsAdjusting() ) {
            return;
        }

        final T baustein =  aktualisiereInformationen();

        setGewaehlterBaustein( baustein );
    }

    /**
     * Prueft ob ein Baustein den Vorteil Vollzauberer vergibt.
     *
     * @param baustein
     *            der ueberprueft werden soll.
     * @return {@code true}, wenn der Baustein den Vorteil Vollzauberer
     *          vergibt, sonst {@code false}.
     */
    final boolean vergibtVollzauberer( final Heldenbaustein baustein ) {

        final List< VorNachteilBonus > boni = baustein.getVorteile();

        final VorNachteilWert vollzauberer = new VorNachteilWert(
                VorNachteil.VOLLZAUBERER );

        for ( final VorNachteilBonus bonus : boni ) {

            final List< VorNachteilWert > werte = bonus.getWerte();

            if ( werte.contains( vollzauberer ) ) {

                return true;
            }
        }

        return false;
    }

    /**
     * Prueft ob ein Baustein den Vorteil Viertelzauberer vergibt.
     *
     * @param baustein
     *            der ueberprueft werden soll.
     * @return {@code true}, wenn der Baustein den Vorteil Viertelzauberer
     *         vergibt, sonst {@code false}
     */
    final boolean vergibtViertelzauberer( final Heldenbaustein baustein ) {

        final List< VorNachteilBonus > boni = baustein.getVorteile();

        final VorNachteilWert viertelzauberer1 = new VorNachteilWert(
                VorNachteil.VIERTELZAUBERER );

        final VorNachteilWert viertelzauberer2 = new VorNachteilWert(
                VorNachteil.VIERTELZAUBERER_UNBEWUSST );

        for ( final VorNachteilBonus bonus : boni ) {

            final List< VorNachteilWert > werte = bonus.getWerte();

            if ( werte.contains( viertelzauberer1 )
              || werte.contains( viertelzauberer2 ) ) {

                return true;
            }
        }

        return false;
    }

    public void update(final Observable o, final Object arg) {

        if ( bausteineGeaendert( arg ) ) {

            bausteinliste.setListData( getBausteine() );

            setGewaehlterBaustein( null );

        } else {

            aktualisiereInformationen();

        }
    }
}
