/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.GroupLayout.ParallelGroup;
import org.jdesktop.layout.GroupLayout.SequentialGroup;

import de.bernhardjung.dsaprogramm.basis.Wuerfel;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.util.AlterFormel;
import de.bernhardjung.dsaprogramm.bausteine.util.GewichtFormel;
import de.bernhardjung.dsaprogramm.bausteine.util.GroesseFormel;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungZwergin;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Erlaubt es verschiedene Einstellungen fuer eine Rasse zu treffen Name,
 * Groesse, LE-Mod. usw..
 *
 * @author Bernhard Jung
 */
class PanelMetaRasseDiverses extends JPanel {

    /**
     * Der Abstand in Pixeln, der zwischen einzelnen Elementen der
     * Benutzeroberflaeche, bestehen soll.
     */
    private static final int ABSTAND_HORIZONTAL = 5;

    /**
     * Der vertikale Abstand in Pixeln, der zwischen einzelnen Elementen der
     * Benutzeroberflaeche, bestehen soll.
     */
    private static final int ABSTAND_VERTIKAL = 10;

    /** Der Text neben der Formel fuer das Alter. */
    private static final String ALTER =
        I18n.i18n( "PanelMetaRasseDiverses.Alter" );

    /**
     * Der Text fuer den Knopf zum Berechnen der GP-Kosten.
     */
    private static final String BERECHNEN =
        I18n.i18n( "PanelMetaKulturDiverses.Berechnen" );

    /** Der Text neben der Formel fuer das Gewicht. */
    private static final String GEWICHT =
        I18n.i18n( "PanelMetaRasseDiverses.Gewicht" );

    /** Der Text neben der Formel fuer die Groesse. */
    private static final String GROESSE =
        I18n.i18n( "PanelMetaRasseDiverses.Groesse" );

    /** Der Text neben der Textifeld fuer die Id. */
    private static final String ID =
        I18n.i18n( "PanelMetaRasseDiverses.Id" );

    /** Der Text neben dem Spinner fuer die Kosten. */
    private static final String KOSTEN =
        I18n.i18n( "PanelMetaRasseDiverses.Kosten" );

    /** Der Text neben dem Textfeld fuer den Namen. */
    private static final String NAME =
        I18n.i18n( "PanelMetaRasseDiverses.Name" );

    /** Zwerginnen benoetigen MU mindestens 12. */
    private static final String ZWERGIN =
        I18n.i18n( "PanelMetaRasseDiverses.Zwergin" );

    /** Der Spinner mit dem die Basisjahre fuer das Alter bestimmt werden. */
    private final JSpinner alterBasis;

    /**
     * Legt den Wuerfelwurf fest, der zu den Basisjahren fuer das Alter
     * addiert wird.
     */
    private final JComboBox alterWuerfel;

    /**
     * Die Zahl die von der Koerpergroesse abgezogen wird um das Gewicht zu
     * bestimmen.
     */
    private final JSpinner gewichtAbzug;

    /** Die Basisgroesse in Zentimetern. */
    private final JSpinner groesseBasis;

    /** Die Anzahl W20 die zur Basisgroesse addiert werden. */
    private final JSpinner groesseAnzahlW20;

    /** Die Anzahl W6 die zur Basisgroesse addiert werden. */
    private final JSpinner groesseAnzahlW6;

    /** Das Textfeld fuer die Id. */
    private final JTextField id;

    /** Der Spinner mit dem die Kosten der Rasse angegeben werden. */
    private final JSpinner kosten;

    /** Das Textfeld, in das der Name der Rasse eingegeben wird. */
    private final JTextField name;

    private final PanelMetaRasse panelRasse;

    /**
     * Zwerginnen, die ihre Sippe verlassen muessen MU mindestens 12 besitzen.
     */
    private final JCheckBox zwergin;

    PanelMetaRasseDiverses( final PanelMetaRasse dasPanelRasse ) {

        alterBasis = Util.erzeugeSpinner( 16, 0, 999, 1 );

        alterWuerfel = new JComboBox( Wuerfel.getWuerfel() );

        gewichtAbzug = Util.erzeugeSpinner( 100, 0, 999, 1 );

        groesseBasis = Util.erzeugeSpinner( 160, 0, 999, 1 );

        groesseAnzahlW20 = Util.erzeugeSpinner( 2, 0, 99, 1 );

        groesseAnzahlW6 = Util.erzeugeSpinner( 0, 0, 99, 1 );

        id = new JTextField();

        kosten = Util.erzeugeSpinner( 0, 0, 999, 1 );

        name = new JTextField();

        panelRasse = dasPanelRasse;

        zwergin = new JCheckBox( ZWERGIN );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Uebernimmt die Werte einer Rasse in die Benutzeroberflaeche.
     *
     * @param rasse
     *            deren Werte angezeigt werden sollen.
     */
    public final void anzeigen( final Rasse rasse ) {

        // Name
        // ****
        name.setText( rasse.getName() );

        // Id
        // **
        final String idString = rasse.getId();

        final int index = idString.indexOf( '.' );

        final String suffix = idString.substring( 1 + index );

        id.setText( suffix );

        // Kosten
        // ******
        kosten.setValue( ( (KostenFest)rasse.getKostenObjekt() ).getGpKosten());

        // Alter
        // *****
        final AlterFormel alter = rasse.getAlter();

        alterBasis.setValue( alter.getBasis() );

        alterWuerfel.setSelectedItem( alter.getWuerfel() );

        // Groesse
        // *******
        final GroesseFormel groesse = rasse.getGroesse();

        groesseBasis.setValue( groesse.getBasis() );

        groesseAnzahlW20.setValue( groesse.getAnzahlW20() );

        groesseAnzahlW6.setValue( groesse.getAnzahlW6() );

        // Gewicht
        // *******
        final GewichtFormel gewicht = rasse.getGewicht();

        gewichtAbzug.setValue( gewicht.getAbzug() );

        // Voraussetzung Zwerginnen
        // ************************
        final List< Voraussetzung > voraussetzungen = rasse.getVoraussetzungen();

        zwergin.setSelected( voraussetzungen.size() > 0 );
    }

    private void aufbauenBenutzeroberflaeche() {

        final GroupLayout layout = new GroupLayout( this );

        setLayout( layout );

        layout.setAutocreateContainerGaps( true );

        final ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup(hGroup);

        final SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        // Zeile 1
        // *******
        final JLabel lName = new JLabel( NAME );

        hGroup.add( layout.createSequentialGroup()
                .add( lName )
                .add( ABSTAND_HORIZONTAL )
                .add( name ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( lName )
                .add( name ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 1.1
        // **********
        final JLabel lId = new JLabel( ID );

        hGroup.add( layout.createSequentialGroup()
                .add( lId )
                .add( ABSTAND_HORIZONTAL )
                .add( id ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( lId )
                .add( id ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 2
        // *******
        final JLabel lKosten = new JLabel( KOSTEN );

        final JLabel kostenLabel = new JLabel();

        final JButton berechnen = new JButton( BERECHNEN );
        berechnen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {

                final Rasse rasse = new Rasse( "Rasse" );

                panelRasse.uebertrageDaten( rasse );

                final double kosten = rasse.berechneGpKosten();

                kostenLabel.setText( String.format( "%1$.1f", kosten ) );
            }
        });

        hGroup.add( layout.createSequentialGroup()
                .add( lKosten )
                .add( ABSTAND_HORIZONTAL )
                .add( kosten,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE )
                .add( ABSTAND_HORIZONTAL )
                .add( berechnen )
                .add( ABSTAND_HORIZONTAL )
                .add( kostenLabel ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( lKosten )
                .add( kosten )
                .add( berechnen )
                .add( kostenLabel ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 3
        // *******
        final JLabel lAlter = new JLabel( ALTER );

        JLabel plus1 = new JLabel( "+" );

        hGroup.add( layout.createSequentialGroup()
                .add( lAlter )
                .add( ABSTAND_HORIZONTAL )
                .add( alterBasis,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE )
                .add( ABSTAND_HORIZONTAL )
                .add( plus1 )
                .add( ABSTAND_HORIZONTAL )
                .add( alterWuerfel,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE  ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( lAlter )
                .add( alterBasis )
                .add( plus1 )
                .add( alterWuerfel ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 4
        // *******
        final JLabel lGroesse = new JLabel( GROESSE );

        plus1 = new JLabel( "+" );

        final JLabel lW6 = new JLabel( Wuerfel.W6.toString() );

        final JLabel plus2 = new JLabel( "+" );

        final JLabel lW20 = new JLabel( Wuerfel.W20.toString() );

        hGroup.add( layout.createSequentialGroup()
                .add( lGroesse )
                .add( ABSTAND_HORIZONTAL )
                .add( groesseBasis,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE )
                .add( ABSTAND_HORIZONTAL )
                .add( plus1 )
                .add( ABSTAND_HORIZONTAL )
                .add( groesseAnzahlW20,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE  )
                .add( ABSTAND_HORIZONTAL / 2 )
                .add( lW20 )
                .add( ABSTAND_HORIZONTAL )
                .add( plus2 )
                .add( ABSTAND_HORIZONTAL )
                .add( groesseAnzahlW6,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE  )
                .add( ABSTAND_HORIZONTAL / 2 )
                .add( lW6 ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( lGroesse )
                .add( groesseBasis )
                .add( plus1 )
                .add( groesseAnzahlW20 )
                .add( lW20 )
                .add( plus2 )
                .add( groesseAnzahlW6 )
                .add( lW6 ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 5
        // *******
        final JLabel lGewicht = new JLabel( GEWICHT );

        final JLabel lGroesseMinus = new JLabel( GROESSE + " -" );

        hGroup.add( layout.createSequentialGroup()
                .add( lGewicht )
                .add( ABSTAND_HORIZONTAL )
                .add( lGroesseMinus )
                .add( ABSTAND_HORIZONTAL )
                .add( gewichtAbzug,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( lGewicht )
                .add( lGroesseMinus )
                .add( gewichtAbzug ) );

        // Zeilenabstand
        // *************
        einfuegenZeilenAbstand( hGroup, vGroup );

        // Zeile 6
        // *******
        hGroup.add( layout.createSequentialGroup()
                .add( zwergin ) );

        vGroup.add( layout.createParallelGroup( GroupLayout.BASELINE, false )
                .add( zwergin ) );

        layout.linkSize( new Component[]{
                lName,
                lId,
                lKosten,
                lAlter,
                lGroesse,
                lGewicht } );
    }

    private void einfuegenZeilenAbstand(
            final ParallelGroup hGroup,
            final SequentialGroup vGroup ) {

        final Component zeilenabstand =
            Box.createVerticalStrut( ABSTAND_VERTIKAL );

        hGroup.add( zeilenabstand );

        vGroup.add( zeilenabstand );
    }

    /** @return Die fuer die Rasse gewaehlte Id. */
    public String getRassenId() {
        return "Rasse." + id.getText().trim();
    }

    /**
     * @return den Namen, den der Benutzer fuer die Rasse angegeben hat.
     */
    public String getRassenName() {
        return name.getText();
    }

    /** Loescht die Eintraege im Panel. */
    public void reset() {
        id.setText( "" );

        kosten.setValue( 0 );

        name.setText( "" );

        zwergin.setSelected( false );
    }

    /**
     * Stellt die Werte einer Rasse, so ein wie sie in der Benutzeroberflaeche
     * gewaehlt sind.
     *
     * @param rasse
     *            die die Werte erhalten soll.
     */
    @SuppressWarnings( "unchecked" )
    public final void uebertrageDaten( final Rasse rasse ) {

        rasse.setName( name.getText() );

        rasse.setKosten( new KostenFest( (Integer) kosten.getValue() ) );

        final AlterFormel alter =
            new AlterFormel( (Integer) alterBasis.getValue(),
                             (Wuerfel )alterWuerfel.getSelectedItem() );

        rasse.setAlter( alter );

        final GroesseFormel groesse =
            new GroesseFormel( (Integer) groesseBasis.getValue(),
                               (Integer) groesseAnzahlW20.getValue(),
                               (Integer) groesseAnzahlW6.getValue() );

        rasse.setGroesse( groesse );

        rasse.setGewicht(
                new GewichtFormel( (Integer) gewichtAbzug.getValue() ) );

        if ( zwergin.isSelected() ) {

            rasse.setVoraussetzungen( Arrays.asList(
                    (Voraussetzung) new VoraussetzungZwergin() ) );

        } else {

            rasse.setVoraussetzungen( Collections.EMPTY_LIST );
        }
    }
}
