/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.GroupLayout.ParallelGroup;
import org.jdesktop.layout.GroupLayout.SequentialGroup;

import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Beobachter;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

abstract class PanelMetaVorNachteileAbstrakt
extends JPanel
implements Beobachter {

    /** Zeigt die GP-Kosten eines Vor-/Nachteils an.*/
    private static final String GP =
        I18n.i18n( "PanelMetaVorNachteileAbstrakt.Gp" );

    private final Map< VorNachteil, List< Integer > > abhaengigkeiten;

    /**
     * Wird aufgerufen, wenn auf eine Checkbox geklickt wird.
     */
    private final ActionListener actionAnAbwaehlen;

    /**
     * Wird aufgerufen, wenn eine Variante gewaehlt wird.
     */
    private final ActionListener actionVariante;

    /**
     * Die Liste der Elemente, die der Benutzer ausgewaehlt hat.
     */
    final BeobachtbareListe< VorNachteilWert > gewaehlte;

    /** Die Label, die die GP-Kosten der Vor-/Nachteile anzeigen. */
    private final List< JLabel > kosten;

    /**
     * Wird aufgerufen, wenn die Stufe eines Wertes geaendert wird.
     */
    private final ChangeListener listenerStufe;

    /**
     * Die Spinner mit denen sich die Stufen der Vor-/Nachteile einstellen
     * lassen. Der Integer Wert gibt die Zeile an in der der Spinner steht.
     */
    private final Map< Integer, JSpinner > stufen;

    /**
     * Die Comboboxen fuer die Varianten. Der Integer-Wert gibt die Zeile an
     * in der die Box steht.
     */
    private final Map< Integer, JComboBox > varianten;

    /** Die Checkboxen um einen Wert zu waehlen. */
    private final List< JCheckBox > vorNachteile;

    /**
     * Die Liste der Werte die vom Panel angezeigt werden.
     */
    final List< VorNachteilWert > werte;

    /**
     * @param liste Die Liste der Werte, die der Benutzer ausgewaehlt hat.
     */
    PanelMetaVorNachteileAbstrakt(
            final BeobachtbareListe< VorNachteilWert > liste ) {

        abhaengigkeiten = new HashMap< VorNachteil, List<Integer> >();

        actionAnAbwaehlen = new ActionListener() {

            public void actionPerformed( final ActionEvent e ) {

                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                wertAnAbgewaehlt( index, checkBox.isSelected() );
            }

        };

        actionVariante = new ActionListener() {

            public void actionPerformed( final ActionEvent e ) {

                final JComboBox comboBox = (JComboBox) e.getSource();

                final int index = Integer.valueOf( comboBox.getName() );

                varianteGewaehlt( index, comboBox.getSelectedItem() );
            }

        };

        gewaehlte = liste;
        gewaehlte.anmelden( this );

        kosten = new ArrayList< JLabel >();

        listenerStufe = new ChangeListener() {

            public void stateChanged( final ChangeEvent e ) {

                final JSpinner spinner = (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                stufeGeaendert( index, (Integer) spinner.getValue() );
            }

        };

        stufen = new HashMap< Integer, JSpinner >();

        varianten = new HashMap< Integer, JComboBox >();

        vorNachteile = new ArrayList< JCheckBox >();

        werte = new ArrayList< VorNachteilWert >();
    }

    void aktualisieren( final int index ) {

        final VorNachteilWert wert = werte.get( index );

        final JCheckBox checkBox = vorNachteile.get( index );

        checkBox.setSelected( gewaehlte.contains( wert ) );

        if ( varianten.containsKey( index ) ) {

            varianten.get( index ).setSelectedItem( wert.getVariante() );
        }

        if ( stufen.containsKey( index ) ) {

            stufen.get( index ).setValue( wert.getStufe() );
        }

        kosten.get( index ).setText(
                String.format( GP, wert.getGpKosten() ) );
    }

    void aktualisieren( final VorNachteil vorNachteil ) {

        if ( abhaengigkeiten.containsKey( vorNachteil ) ) {

            for ( final int index : abhaengigkeiten.get( vorNachteil ) ) {

                aktualisieren( index );
            }
        }
    }

    void aufbauenBenutzeroberflaeche() {

        removeAll();

        kosten.clear();
        stufen.clear();
        varianten.clear();
        vorNachteile.clear();

        initAbhaengigkeiten();

        // Das neue Layout festlegen.
        final GroupLayout layout = new GroupLayout( this );

        layout.setAutocreateContainerGaps( true );

        layout.setAutocreateGaps( true );

        setLayout( layout );

        final ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup( hGroup );

        final SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        // Alle 5 Zeilen wird ein Trennstrich eingefuegt, das erleichtert dem
        // Benutzer die Orientierung in den Zeilen der Liste.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // Jetzt die Liste Zeile fuer Zeile aufbauen.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            final SequentialGroup zeileH = layout.createSequentialGroup();
            final ParallelGroup zeileV =
                layout.createParallelGroup( GroupLayout.CENTER, false );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.add( separator );
                zeileV.add( separator );

                hGroup.add( zeileH );
                vGroup.add( zeileV );

                ++trennstriche;

                continue;
            }

            // Das ist der index des Wertes in werte, der in der aktuellen
            // Zeile dargestellt wird.
            final int index = i - trennstriche;

            final String indexString = String.valueOf( index );

            final VorNachteilWert wert = werte.get( index );

            final VorNachteil vorNachteil = wert.getVorNachteil();

            // Spalte mit der Checkbox und evtl mit der Combobox aufbauen.
            if ( vorNachteil.hatVarianten() ) {

                // Hat der Wert eine Variante wird neben der Checkbox
                // eine Combobox benoetigt um die Variante waehlen zu koennen.

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText( vorNachteil.getPraefix() );

                checkBox.setSelected( gewaehlte.contains( wert ) );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionAnAbwaehlen );

                vorNachteile.add( checkBox );

                zeileH.add( checkBox );
                zeileV.add( checkBox );

                final Vector< Object > variantenVector =
                    new Vector< Object >( vorNachteil.getVarianten( null ) );
                final JComboBox comboBox =
                    new JComboBox( variantenVector );

                comboBox.setEditable( vorNachteil.hatFreieVarianten() );

                comboBox.setSelectedItem( wert.getVariante() );

                comboBox.addActionListener( actionVariante );

                varianten.put( index, comboBox );

                comboBox.setName( indexString );

                zeileH.add( comboBox, 100, 100, Short.MAX_VALUE );
                zeileV.add( comboBox );

            } else {

                final JCheckBox checkBox = new JCheckBox( wert.getName() );

                checkBox.setSelected( gewaehlte.contains( wert ) );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionAnAbwaehlen );

                vorNachteile.add( checkBox );

                zeileH.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        Short.MAX_VALUE );
                zeileV.add( checkBox );

            }

            if ( vorNachteil.hatStufen() ) {

                final int stufe = wert.getStufe();

                final int schrittweite = vorNachteil.getSchrittweite();

                final JSpinner spinner =
                    Util.erzeugeSpinner( stufe, schrittweite, 99, schrittweite );

                spinner.addChangeListener( listenerStufe );

                spinner.setName( String.valueOf( index ) );
                stufen.put( index, spinner );

                zeileH.add( spinner,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
                zeileV.add( spinner );
            }

            final JLabel k = Util.erzeugeLabelRechtsbuendig();

            k.setText( String.format( GP, wert.getGpKosten() ) );

            kosten.add( k );

            zeileH.add( k,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            zeileV.add( k );

            hGroup.add( zeileH );
            vGroup.add( zeileV );
        }

        if ( 0 < werte.size() ) {

            layout.linkSize( kosten.toArray( new Component[ kosten.size() ] ) );
        }

    }

    abstract void wertAnAbgewaehlt( final int index, final boolean gewaehlt );

    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final VorNachteil vorNachteil = werte.get( i ).getVorNachteil();

            if ( ! abhaengigkeiten.containsKey( vorNachteil ) ) {

                abhaengigkeiten.put( vorNachteil, new ArrayList< Integer >( 2 ) );
            }

            abhaengigkeiten.get( vorNachteil ).add( i );
        }
    }

    /**
     * Wird aufgerufen, wenn ueber eine Spinner die Stufe eines Wertes
     * veraendert wird.
     *
     * @param index
     *            des Wertes, dessen Stufe geaendert wurde.
     * @param stufe
     *            die fuer den Wert gewaehlt wurde.
     */
    public abstract void stufeGeaendert( final int index, final int stufe );

    /**
     * Wird aufgerufen, wenn in einer Combobox einen Variante gewaehlt wird.
     *
     * @param index
     *            des Wertes fuer den die Variante gewaehlt wurde.
     * @param variante
     *            die gewaehlt wurde.
     */
    public final void varianteGewaehlt( final int index, final Object variante ) {

        if ( null != variante ) {

            final VorNachteilWert wert = werte.get( index );

            wert.setVariante( variante );
        }

        aktualisieren( index );
    }

}
