/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Eine Voraussetzung fuer eine Sonderfertigkeit, die ein Held erfuellen muess
 * wenn er die Sonderfertigkeit waehlen/nutzen will.
 *
 * @author Bernhard Jung
 */
public interface Voraussetzung {

    /**
     * Eine Voraussetzung bei der der Helden die Sonderfertigkeit Ausweichen I
     * besitzen muss um sie zu erfuellen.
     */
    public static final Voraussetzung AUSWEICHEN_I =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.AUSWEICHEN_I );

    /**
     * Eine Voraussetzung bei der der Helden die Sonderfertigkeit Ausweichen II
     * besitzen muss um sie zu erfuellen.
     */
    public static final Voraussetzung AUSWEICHEN_II =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.AUSWEICHEN_II );

    /**
     * Eine Voraussetzung bei der der Helden die Sonderfertigkeit Aufmerksamkeit
     * besitzen muss um sie zu erfuellen.
     */
    public static final Voraussetzung AUFMERKSAMKEIT =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.AUFMERKSAMKEIT );

    /**
     * Die Voraussetzung bei der der Held das Talent Fahrzeuge Lenken auf einer#
     * Stufe von mindestens 7 besitzen muss.
     */
    public static final Voraussetzung FAHRZEUGE_LENKEN_7 =
        new VoraussetzungTalent( Talent.FAHRZEUG_LENKEN, 7 );

    /**
     * Eine Voraussetzung bei der der Helden die Sonderfertigkeit Finte
     * besitzen muss um sie zu erfuellen.
     */
    public static final Voraussetzung FINTE =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.FINTE );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Fingerfertigkeit
     * mindestens 10 betragen muss.
     */
    public static final Voraussetzung FF_10 =
        new VoraussetzungEigenschaft( Eigenschaft.FF, 10 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Fingerfertigkeit
     * mindestens 12 betragen muss.
     */
    public static final Voraussetzung FF_12 =
        new VoraussetzungEigenschaft( Eigenschaft.FF, 12 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Gewandtheit
     * mindestens 10 betragen muss.
     */
    public static final Voraussetzung GE_10 =
        new VoraussetzungEigenschaft( Eigenschaft.GE, 10 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Gewandtheit
     * mindestens 12 betragen muss.
     */
    public static final Voraussetzung GE_12 =
        new VoraussetzungEigenschaft( Eigenschaft.GE, 12 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Gewandtheit
     * mindestens 15 betragen muss.
     */
    public static final Voraussetzung GE_15 =
        new VoraussetzungEigenschaft( Eigenschaft.GE, 15 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Intuition mindestens
     * 10 betragen muss.
     */
    public static final Voraussetzung IN_10 =
        new VoraussetzungEigenschaft( Eigenschaft.IN, 10 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Intuition mindestens
     * 12 betragen muss.
     */
    public static final Voraussetzung IN_12 =
        new VoraussetzungEigenschaft( Eigenschaft.IN, 12 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Intuition mindestens
     * 13 betragen muss.
     */
    public static final Voraussetzung IN_13 =
        new VoraussetzungEigenschaft( Eigenschaft.IN, 13 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Intuition mindestens
     * 15 betragen muss.
     */
    public static final Voraussetzung IN_15 =
        new VoraussetzungEigenschaft( Eigenschaft.IN, 15 );

    /**
     * Eine Voraussetzung bei der die Stufe des Initiative-Basiswerts des
     * Helden mindestens 10 betragen muss.
     */
    public static final Voraussetzung INITIATIVE_10 =
        new VoraussetzungBasiswert( Basiswert.INI, 10 );

    /**
     * Eine Voraussetzung bei der der Helden die Sonderfertigkeit Kampfreflexe
     * besitzen muss um sie zu erfuellen.
     */
    public static final Voraussetzung KAMPFREFLEXE =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.KAMPFREFLEXE );

    /**
     * Eine Voraussetzung bei der der Helden das Talent Koerperbeherrschung
     * auf einer Stufe von mindestens 7 besitzen muss um sie zu erfuellen.
     */
    public static final Voraussetzung KOERPERBEHERRSCHUNG_7 =
        new VoraussetzungTalent( Talent.KOERPERBEHERRSCHUNG, 7 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Koerperkraft
     * mindestens 10 betragen muss.
     */
    public static final Voraussetzung KK_10 =
        new VoraussetzungEigenschaft( Eigenschaft.KK, 10 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Koerperkraft
     * mindestens 12 betragen muss.
     */
    public static final Voraussetzung KK_12 =
        new VoraussetzungEigenschaft( Eigenschaft.KK, 12 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Koerperkraft
     * mindestens 15 betragen muss.
     */
    public static final Voraussetzung KK_15 =
        new VoraussetzungEigenschaft( Eigenschaft.KK, 15 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Klugheit mindestens
     * 10 betragen muss.
     */
    public static final Voraussetzung KL_10 =
        new VoraussetzungEigenschaft( Eigenschaft.KL, 10 );

    /**
     * Eine Voraussetzung bei der die Stufe des Helden in Klugheit mindestens
     * 12 betragen muss.
     */
    public static final Voraussetzung KL_12 =
        new VoraussetzungEigenschaft( Eigenschaft.KL, 12 );

    /** Die Voraussetzung der Sonderfertigkeit Kulturkunde. */
    public static final Voraussetzung KULTURKUNDE =
        new VoraussetzungKulturkunde();

    /**
     * Die Voraussetzung bei der der Held die Sonderfertigkeit Linkhand
     * besitzen muss.
     */
    public static final Voraussetzung LINKHAND =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.LINKHAND );

    /**
     * Die Voraussetzung bei der der Held die Sonderfertigkeit Meisterparde
     * besitzen muss.
     */
    public static final Voraussetzung MEISTERPARADE =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.MEISTERPARADE );

    /**
     * Fuer Sonderfertigkeiten die zu Spielbeginn nicht gewaehlt werden
     * koennen.
     */
    public static final Voraussetzung NICHT_BEGINN =
        new VoraussetzungNichtBeginn();

    /**
     * Fuer die Sonderfertigkeiten des beidhaendigen Kampfes.
     */
    public static final Voraussetzung NICHT_EINARMIG =
        new VoraussetzungNichtVorNachteil( VorNachteil.EINARMIG );

    /** Fuer Sonderfertigkeiten, die nicht frei waehlbar sind. */
    public static final Voraussetzung NICHT_FREI_WAEHLBAR =
        new VoraussetzungNichtFreiWaehlbar();

    /**
     * Die Voraussetzung bei der der Parade-Basiswert des Helden auf mindestens
     * 8 sein muss.
     */
    public static final Voraussetzung PARADE_8 =
        new VoraussetzungBasiswert( Basiswert.PAB, 8 );

    /**
     * Die Voraussetzung bei der der Held das Talent Raufen auf einer Stufe von
     * mindestens 7 besitzen muss.
     */
    public static final Voraussetzung RAUFEN_7 =
        new VoraussetzungTalent( Talent.RAUFEN, 7 );

    /**
     * Eine Voraussetzung bei der die Sonderfertigkeit Regeneration I
     * vorausgesetzt wird.
     */
    public static final Voraussetzung REGENERATION_I =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.REGENERATION_I );

    /**
     * Die Voraussetzung bei der der Held das Talent Reiten auf einer Stufe von
     * mindestens 7 besitzen muss.
     */
    public static final Voraussetzung REITEN_7 =
        new VoraussetzungTalent( Talent.REITEN, 7 );

    /**
     * Die Voraussetzung bei der der Held das Talent Ringen auf einer Stufe von
     * mindestens 7 besitzen muss.
     */
    public static final Voraussetzung RINGEN_7 =
        new VoraussetzungTalent( Talent.RINGEN, 7 );

    /**
     * Die Voraussetzung bei der der Held das Talent Ritualkenntnis auf einer
     * Stufe von mindestens 3 besitzen muss.
     */
    public static final Voraussetzung RITUALKENNTNIS_3 =
        new VoraussetzungTalent( Talent.RITUALKENNTNIS, 3 );

    /**
     * Die Voraussetzung bei der der Held das Talent Ritualkenntnis auf einer
     * Stufe von mindestens 5 besitzen muss.
     */
    public static final Voraussetzung RITUALKENNTNIS_5 =
        new VoraussetzungTalent( Talent.RITUALKENNTNIS, 5 );

    /**
     * Die Voraussetzung bei der der Held das Talent Ritualkenntnis auf einer
     * Stufe von mindestens 7 besitzen muss.
     */
    public static final Voraussetzung RITUALKENNTNIS_7 =
        new VoraussetzungTalent( Talent.RITUALKENNTNIS, 7 );

    /**
     * Die Voraussetzung bei der der Held das Talent Ritualkenntnis auf einer
     * Stufe von mindestens 11 besitzen muss.
     */
    public static final Voraussetzung RITUALKENNTNIS_11 =
        new VoraussetzungTalent( Talent.RITUALKENNTNIS, 11 );

    /**
     * Die Voraussetzung bei der der Held die Sonderfertigkeit
     * Ruestungsgewoehnung I besitzen muss.
     */
    public static final Voraussetzung RUESTUNGSGEWOEHNUNG_I =
        new VoraussetzungSonderfertigkeit(
                Sonderfertigkeit.RUESTUNGSGEWOEHNUNG_I );

    /**
     * Die Voraussetzung bei der der Held die Sonderfertigkeit Schildkampf I
     * besitzen muss.
     */
    public static final Voraussetzung SCHILDKAMPF_I =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.SCHILDKAMPF_I );

    /** Die Voraussetzung fuer Talent- und Waffenspezialisierungen. */
    public static final Voraussetzung SPEZIALISIERUNG =
        new VoraussetzungSpezialisierung();

    /**
     * Die Standard-Voraussetzung fuer alle Sonderfertigkeiten, sie prueft
     * ob der Held eine durch Rasse, Kultur oder Profession vorgegebene
     * Sonderfertigkeit auch besitzt.
     */
    public static final Voraussetzung STANDARD = new VoraussetzungStandard();

    /**
     * Voraussetzung fuer magische Sonderfertigkeiten, die Vollzauberer oder
     * Viertelzauberer benoetigen. */
    public static final Voraussetzung VZ =
        new VoraussetzungVorteil( VorNachteil.VIERTELZAUBERER,
                                  VorNachteil.VOLLZAUBERER );

    /** Voraussetetzung fuer magische Vorteile, die Vollzauberer benoetigen. */
    public static final Voraussetzung Z =
        new VoraussetzungVorteil( VorNachteil.VOLLZAUBERER );

    /**
     * Die Voraussetzung bei der der Held die Sonderfertigkeit Wuchtschlag
     * besitzen muss.
     */
    public static final Voraussetzung WUCHTSCHLAG =
        new VoraussetzungSonderfertigkeit( Sonderfertigkeit.WUCHTSCHLAG );

    /**
     * Die Voraussetzung bei der der Held den Vorteil Zweistimmiger Gesang
     * besitzen muss.
     */
    public static final Voraussetzung ZWEISTIMMIGER_GESANG =
        new VoraussetzungVorteil( VorNachteil.ZWEISTIMMIGER_GESANG );

    /**
     * Liefert die Regelelemente (Talent, VorNachteil, usw ) die fuer die
     * Ueberpruefung der Voraussetzung relevant sind.
     *
     * @return die Regelelemente von denen die Voraussetzung abhaengig ist.
     */
    public Collection< Object > getAbhaengigkeiten();

    /**
     * Prueft fuer einen Held ob er die Voraussetzung fuer den uebergebenen
     * Wert erfuellt.
     *
     * @param held fuer den die Voraussetzung geprueft werden soll.
     * @param wert den der Held waehlen will oder schon besitzt.
     * @return eine Beschreibung aller auftretenden Regelprobleme, wenn der
     *      Held den uebergebenen Wert waehlen will oder schon besitzt.
     */
    public KonfliktBeschreibung pruefen(
            final Held held,
            final SonderfertigkeitWert wert );
}
