/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Eine Voraussetzung bei der der Held einen Basiswert auf eine Mindeststufe
 * haben muss.
 * 
 * @author Bernhard Jung
 */
class VoraussetzungBasiswert implements Voraussetzung {

    /** Eine Beschreibung der Voraussetzung fuer den Benutzer. */
    private final String beschreibungString;
    
    /** Der Basiswert in dem die Mindeststufe erreicht werden muss. */
    private final Basiswert basiswert;

    /** Die Mindeststufe die in dem Basiswert erreicht werden muss. */
    private final int mindeststufe;
    
    VoraussetzungBasiswert( 
            final Basiswert derBasiswert,
            final int dieMindeststufe ) {
        
        basiswert = derBasiswert;
        
        mindeststufe = dieMindeststufe;
        
        beschreibungString = String.format(
                I18n.i18n( "VoraussetzungBasiswert" ),
                basiswert.getKuerzel(),
                mindeststufe );
        
    }
    
    /** 
     * @return den Basiswert und die zum Berechnen des Basiswerts benoetigten
     *         Eigenschaften.
     */
    public Collection<Object> getAbhaengigkeiten() {
        
        final List< Object > abhaengigkeiten = new ArrayList< Object >();
        
        abhaengigkeiten.add( basiswert );
        
        abhaengigkeiten.addAll( basiswert.getFormelEigenschaften() );
        
        return abhaengigkeiten;
    }
    
    /**
     * Prueft ob der Held die Mindeststufe in dem Basiswert besitzt.
     */
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final SonderfertigkeitWert wert) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();
        
        final int stufe = held.getStufe( basiswert );
        
        if ( stufe < mindeststufe ) {
            
            beschreibung.einfuegen( beschreibungString );
        }
        
        return beschreibung;
    }

}
