/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Konstanten fuer die im Programm verfuegbaren Talentgruppen. Die Talentgruppe
 * Sprachen/Schriften ist im Programm in 2 Talentgruppen aufgespalten. Es gibt
 * eine Talentgruppe Sprachen und eine Talentgruppe Schriften. Es existiert eine
 * Talengruppe fuer die Gaben und eine fuer Ritualkenntnis.
 * <p>
 * Jedes Talent ist einer Talentgruppe zugeordnet. Kein Talent ist zwei
 * Talentgruppen zugeordnet. Eine Talentgruppe ist eine Liste von Talenten.
 *
 * @author Bernhard Jung
 */
public class Talentgruppe
extends AbstractList< Talent >
implements Comparable< Talentgruppe >
{

    /** Eine Liste aller Talentgruppen, ohne die Talengruppe Gaben. */
    private static final List< Talentgruppe > TALENTGRUPPEN;

    /**
     * Eine Liste aller Talentgruppen, einschliesslich der Talentgruppe Gaben
     * und Ritualkenntnis.
     */
    private static final List< Talentgruppe > TALENTGRUPPEN_ALLE;

    /**
     * Eine Map in der jeder Talentgruppe unter ihrer ID gespeichert ist.
     */
    private static final Map< String, Talentgruppe > TALENTGRUPPEN_MAP =
        new HashMap< String, Talentgruppe >();

    /** Die Talentgruppe Kampftechnik-Talente. */
    public static final Talentgruppe KAMPF = new Talentgruppe(
                "Talentgruppe.Kampf", 0 );

    /** Die Talentgruppe Koerperliche Talente. */
    public static final Talentgruppe KOERPER =
        new Talentgruppe( "Talentgruppe.Koerper", 1 );

    /** Die Talentgruppe Gesellschaftliche Talente. */
    public static final Talentgruppe GESELLSCHAFT =
        new Talentgruppe( "Talentgruppe.Gesellschaft", 2 );

    /** Die Talentgruppe Natur-Talente. */
    public static final Talentgruppe NATUR =
        new Talentgruppe( "Talentgruppe.Natur", 3 );

    /** Die Talentgruppe Wissens-Talente. */
    public static final Talentgruppe WISSEN =
        new Talentgruppe( "Talentgruppe.Wissen", 4 );

    /** Die Talentgruppe Sprachen. */
    public static final Talentgruppe SPRACHEN =
        new Talentgruppe( "Talentgruppe.Sprachen", 5 );

    /** Die Talentgruppe Schriften. */
    public static final Talentgruppe SCHRIFTEN =
        new Talentgruppe( "Talentgruppe.Schriften", 6 );

    /** Die Talentgruppe Handwerks-Talente. */
    public static final Talentgruppe HANDWERK =
        new Talentgruppe( "Talentgruppe.Handwerk", 7 );

    /** Die Talentgruppe fuer Gaben. */
    public static final Talentgruppe GABEN =
        new Talentgruppe( "Talentgruppe.Gaben", 8 );

    /** Die Talentgruppe fuer Ritualkenntnis. */
    public static final Talentgruppe RITUALKENNTNIS =
        new Talentgruppe( "Talentgruppe.Ritualkenntnis", 9 );

    static {
        TALENTGRUPPEN = new ArrayList< Talentgruppe >( 8 );
        TALENTGRUPPEN.add( KAMPF );
        TALENTGRUPPEN.add( KOERPER );
        TALENTGRUPPEN.add( GESELLSCHAFT );
        TALENTGRUPPEN.add( NATUR );
        TALENTGRUPPEN.add( WISSEN );
        TALENTGRUPPEN.add( SPRACHEN );
        TALENTGRUPPEN.add( SCHRIFTEN );
        TALENTGRUPPEN.add( HANDWERK );

        TALENTGRUPPEN_ALLE = new ArrayList< Talentgruppe >( 8 );
        TALENTGRUPPEN_ALLE.add( KAMPF );
        TALENTGRUPPEN_ALLE.add( KOERPER );
        TALENTGRUPPEN_ALLE.add( GESELLSCHAFT );
        TALENTGRUPPEN_ALLE.add( NATUR );
        TALENTGRUPPEN_ALLE.add( WISSEN );
        TALENTGRUPPEN_ALLE.add( SPRACHEN );
        TALENTGRUPPEN_ALLE.add( SCHRIFTEN );
        TALENTGRUPPEN_ALLE.add( HANDWERK );
        TALENTGRUPPEN_ALLE.add( GABEN );
        TALENTGRUPPEN_ALLE.add( RITUALKENNTNIS );
    }

    /**
     * @param id
     *            der gesuchten Talentgruppe.
     * @return die Talentgruppe mit der uebergebenen ID.
     * @throws IllegalArgumentException
     *             wenn es keine Talentgruppe mit der uebergebenen ID gibt.
     */
    public static final Talentgruppe getTalentgruppe( final String id ) {

        if ( TALENTGRUPPEN_MAP.containsKey( id ) ) {

            return TALENTGRUPPEN_MAP.get( id );

        } else {

            throw new IllegalArgumentException( "Es existiert keine " +
                    "Talentgruppe mit der Id " + id );
        }
    }

    /**
     * @return eine Liste aller Talentgruppen ohne die Talentgruppe Gaben
     *         und Ritualkenntnis.
     */
    public static final List< Talentgruppe > getTalentgruppen() {
        return new ArrayList< Talentgruppe >( TALENTGRUPPEN );
    }

    /**
     * @return eine Liste aller Talentgruppen mit der Talentgruppe Gaben
     *         und Ritualkenntnis.
     */
    public static final List< Talentgruppe > getTalentgruppenAlle() {
        return new ArrayList< Talentgruppe >( TALENTGRUPPEN_ALLE );
    }

    /** Die Id der Talentgruppe. */
    private final String id;

    /** Der Index zum Sortieren der Talentgruppen. */
    private final int index;

    /** Die Kurzform des Namens der Talengruppe, zB 'Kampf' oder 'Wissen' */
    private final String kuerzel;

    /**
     * Der volle Name der Talentgruppe, zB 'Kampftechnik-Talente' oder
     * 'Wissens-Talente'
     */
    private final String name;

    /** Die Talente dieser Talentgruppe. */
    private final List< Talent > talente;

    /**
     * Erzeugt eine Talentgruppe.
     *
     * @param dieId der Talentgruppe.
     * @param derIndex zum Sortieren der Talentgruppe.
     */
    private Talentgruppe( final String dieId, final int derIndex ) {

        id = dieId;

        index = derIndex;

        name = I18n.i18n( id + ".Name" );

        kuerzel = I18n.i18n( id + ".Kuerzel" );

        talente = new ArrayList< Talent >();

        TALENTGRUPPEN_MAP.put( id, this );
    }

    /**
     * Sortiert die Talentgruppen wie auf dem Charakterbogen angegeben.
     */
    public final int compareTo( final Talentgruppe talentgruppe ) {
        return index - talentgruppe.index;
    }

    /**
     * Fuegt der Talentgruppe ein Talent hinzu.
     *
     * @param talent der Talentgruppe.
     */
    final void einfuegen( final Talent talent ) {

        int index = Collections.binarySearch( talente, talent );

        if ( index < 0 ) {
            // Das Talent ist noch nicht in der Liste.

            index = -( index + 1 );

            talente.add( index, talent );
        }

    }

    /**
     * @param index liefert das Talent an Position index.
     */
    @Override
    public Talent get( final int index ) {
        return talente.get( index );
    }

    /**
     * @return die ID der Talentgruppe.
     */
    public final String getId() {
        return id;
    }

    /** @return den kurzen Namen der Talentgruppe. */
    public final String getKuerzel() {
        return kuerzel;
    }

    /** @return den Namen der Talentgruppe. */
    public final String getName() {
        return name;
    }

    /**
     * @return die Anzahl Talente in der Talentgruppe.
     */
    @Override
    public int size() {
        return talente.size();
    }

    /**
     * @see #getName()
     * @return den Namen der Talentgruppe.
     */
    @Override
    public final String toString() {
        return getName();
    }


}
